/*--------------------------------------------------------------------------+
$Id: SafetyFunctionArchCreator.java 3451 2016-09-02 13:36:29Z diewald $
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.create.safetyfunctionarch;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.safetyfunctionarch.SafetyFunctionArchEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.create.ComposableCreatorBase;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.genotype.FlatGenotypeMap;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.graph.DecoderDependencyGraph;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.FlatPhenotypeMap;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.fortiss.af3.exploration.alg.service.IExplorationEncoding;

import com.google.inject.Inject;

/**
 * Creator for the {@link SafetyFunctionArchEncoding} that provides a standard encoding for the
 * exploration of safety function designs.
 * 
 * @author diewald
 * @author $Author: diewald $
 * @version $Rev: 3451 $
 * @ConQAT.Rating YELLOW Hash: F3DF227B23CDB81D39D60C772E7B5FEE
 */
public class SafetyFunctionArchCreator<C> extends
		ComposableCreatorBase<SafetyFunctionArchEncoding<C>> {

	/** Constructor. */
	@Inject
	public SafetyFunctionArchCreator(SystemModelAdapter<C, ?, ?, ?, ?, ?> systemModelAdapter,
			DecoderDependencyGraph execDepGraph) {
		super(systemModelAdapter, execDepGraph);
	}

	/** {@inheritDoc} */
	@SuppressWarnings({"rawtypes", "unchecked"})
	@Override
	public SafetyFunctionArchEncoding<C> createComposable(FlatGenotypeMap createdGenotypes,
			FlatPhenotypeMap decodedPhenotypes) {
		// It is clear that the generic is right. However, the compiler does not get it. Thus,
		// suppress the corresponding Warnings.
		return new SafetyFunctionArchEncoding(systemModelAdapter.getSafetyFunctionAdapters(), true,
				true);
	}

	/** {@inheritDoc} */
	@Override
	public Class<? extends IExplorationEncoding> getCreatedEncoding() {
		return SafetyFunctionArchEncoding.class;
	}

}
