/*--------------------------------------------------------------------------+
$Id: AbstractTaskMappingCreator.java 3480 2016-09-09 13:31:04Z diewald $
|                                                                          |
| Copyright 2015 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.create.taskmapping;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskgraph.AbstractTaskGraphEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.genotype.FlatGenotypeMap;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.graph.DecoderDependencyGraph;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.FlatPhenotypeMap;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IDeployableComponentAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.fortiss.af3.exploration.alg.exception.ExplorationException;
import org.fortiss.af3.exploration.alg.service.IExplorationEncoding;
import org.fortiss.af3.exploration.moea.model.DseSpecification;
import org.opt4j.core.Genotype;
import org.opt4j.core.problem.Creator;

import com.google.inject.Inject;

/**
 * {@link Creator} for an {@link AbstractTaskMappingEncoding}, i.e. a {@link Genotype} that may
 * refer which need to be instantiated.
 * 
 * @author diewald
 * @author $Author: diewald $
 * @version $Rev: 3480 $
 * @ConQAT.Rating YELLOW Hash: B5E7282821DC0BDD3F21BEEFF206CADB
 */
public class AbstractTaskMappingCreator extends
		TaskMappingCreator<AbstractTaskMappingEntry, AbstractTaskMappingEncoding> {

	/** Constructor */
	@Inject
	public AbstractTaskMappingCreator(DseSpecification dse,
			SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter,
			DecoderDependencyGraph execDepGraph) {
		super(dse, systemModelAdapter, execDepGraph);
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @throws ExplorationException
	 *             if no valid allocation targets can be identified for one
	 *             {@link IDeployableComponentAdapter}.
	 */
	@Override
	public AbstractTaskMappingEncoding createComposable(FlatGenotypeMap createdGenotypes,
			FlatPhenotypeMap<Phenotype> decodedPhenotypes) throws ExplorationException {
		AbstractTaskGraphEncoding<?, ?> atgEnc =
				decodedPhenotypes.getEncoding(AbstractTaskGraphEncoding.class);
		assert (atgEnc != null) : "The required abstract task graph encoding could not be found in the map of already decoded phenotypes.";
		// The AbstractTaskMappingEncoding may be NULL to indicate that no previous step produced a
		// partial mapping.
		AbstractTaskMappingEncoding atmEnc =
				createdGenotypes.get(AbstractTaskMappingEncoding.class);
		return new AbstractTaskMappingEncoding(dse, systemModelAdapter, atgEnc, atmEnc);
	}

	/** {@inheritDoc} */
	@Override
	public Class<? extends IExplorationEncoding> getCreatedEncoding() {
		return AbstractTaskMappingEncoding.class;
	}
}
