/*--------------------------------------------------------------------------+
$Id: InstantiatedTaskGraphEncoding.java 3797 2016-10-28 11:14:06Z diewald $
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskgraph;

import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.InstantiatedTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.InstantiatedTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.ComposablePhenotype;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.taskgraph.DefaultTaskGraph;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.taskgraph.TaskGraph;

/**
 * {@link TaskGraphEncoding} for {@link TaskGraph}s that contain <i>concrete</i> tasks exclusively.
 * 
 * @author diewald
 * @author $Author: diewald $
 * @version $Rev: 3797 $
 * @ConQAT.Rating YELLOW Hash: C03B8040E16131B7125651C59098DC72
 */
public class InstantiatedTaskGraphEncoding<T extends InstantiatedTaskMappingEntry, C> extends
		TaskGraphEncoding<InstantiatedTaskMappingEncoding<T>, C, DefaultTaskGraph> {

	/**
	 * Constructor. See {@link TaskGraphEncoding#TaskGraphEncoding(SystemModelAdapter, Collection)}.
	 */
	public InstantiatedTaskGraphEncoding(SystemModelAdapter<C, ?, ?, ?, ?, ?> systemModelAdapter,
			Collection<DefaultTaskGraph> taskGraphs) {
		super(systemModelAdapter, taskGraphs);
	}

	/** Constructor. See {@link TaskGraphEncoding#TaskGraphEncoding(SystemModelAdapter)}. */
	public InstantiatedTaskGraphEncoding(SystemModelAdapter<C, ?, ?, ?, ?, ?> systemModelAdapter) {
		super(systemModelAdapter);
	}

	/** Copy Constructor. See {@link TaskGraphEncoding#TaskGraphEncoding(TaskGraphEncoding)}. */
	public InstantiatedTaskGraphEncoding(InstantiatedTaskGraphEncoding<T, C> encoding) {
		super(encoding);
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public Class<? extends ComposablePhenotype<?>> getPTypeToRegister() {
		return (Class<? extends ComposablePhenotype<?>>)InstantiatedTaskGraphEncoding.class;
	}
}
