/*--------------------------------------------------------------------------+
$Id: SafeTaskGraphEncoding.java 3797 2016-10-28 11:14:06Z diewald $
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskgraph;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.safetyfunctionarch.SafetyFunctionArchEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.ComposablePhenotype;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.taskgraph.TaskGraph;

/**
 * {@link TaskGraphEncoding} which provides {@link TaskGraph}s for explored architectures of safety
 * functions that are encoded by a {@link SafetyFunctionArchEncoding}.
 * 
 * @author diewald
 * @author $Author: diewald $
 * @version $Rev: 3797 $
 * @ConQAT.Rating YELLOW Hash: 044839861F95037D2DD4D480279F2C8B
 */
public class SafeTaskGraphEncoding<C> extends
		AbstractTaskGraphEncoding<SafetyFunctionArchEncoding<C>, C> {

	/** Constructor. See {@link TaskGraphEncoding#TaskGraphEncoding(SystemModelAdapter)}. */
	public SafeTaskGraphEncoding(SystemModelAdapter<C, ?, ?, ?, ?, ?> systemModelAdapter) {
		super(systemModelAdapter);
	}

	/** Copy Constructor. See {@link TaskGraphEncoding#TaskGraphEncoding(TaskGraphEncoding)}. */
	public SafeTaskGraphEncoding(SafeTaskGraphEncoding<C> encoding) {
		super(encoding);
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public Class<? extends ComposablePhenotype<?>> getPTypeToRegister() {
		return (Class<? extends ComposablePhenotype<?>>)SafeTaskGraphEncoding.class;
	}
}
