/*--------------------------------------------------------------------------+
$Id: TaskMappingEntry.java 3797 2016-10-28 11:14:06Z diewald $
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping;

import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IDeployableComponentAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.mapping.IMappingEntry;

import com.google.common.collect.ClassToInstanceMap;
import com.google.common.collect.MutableClassToInstanceMap;

/**
 * Base class for {@link TaskMappingEntry}s representing the one allocation of an
 * {@link IDeployableComponentAdapter} to an {@link IExecutionUnitAdapter}.
 * 
 * @author barner
 * @author $Author: diewald $
 * @version $Rev: 3797 $
 * @ConQAT.Rating YELLOW Hash: 5CB5AB5FCB8696537D2229C5E5FC15E0
 */
public abstract class TaskMappingEntry implements IMappingEntry {
	/** Deployment target to which task is mapped */
	private IExecutionUnitAdapter<?> target;

	/** {@link IDeployableComponentAdapter} that is "realized" by this {@link TaskMappingEntry}. */
	protected IDeployableComponentAdapter<?> deploymentSource;

	/** Associated {@link IMappingEntry}s that were generated during the DSE process. */
	private ClassToInstanceMap<IMappingEntry> associatedEntries = MutableClassToInstanceMap
			.create();

	/** Constructs a mapping entry the represents the mapping to a given deployment target. */
	protected TaskMappingEntry(IDeployableComponentAdapter<?> deployableComponent,
			IExecutionUnitAdapter<?> target) {
		this.deploymentSource = deployableComponent;
		this.target = target;
	}

	/** {@inheritDoc} */
	@Override
	public IExecutionUnitAdapter<?> getTarget() {
		return target;
	}

	/**
	 * Sets target {@link IExecutionUnitAdapter} to the given {@link IExecutionUnitAdapter}
	 * overriding the previous target reference.
	 * 
	 * @param target
	 *            {@link IExecutionUnitAdapter} defining the allocation target.
	 */
	public void resetTarget(IExecutionUnitAdapter<?> target) {
		this.target = target;
	}

	/** {@inheritDoc} */
	@Override
	public IDeployableComponentAdapter<?> getDeployableComponent() {
		return deploymentSource;
	}

	/** {@inheritDoc} */
	@Override
	public void addAssociatedElement(IMappingEntry assocEntry) {
		associatedEntries.put(assocEntry.getClass(), assocEntry);
	}

	/** {@inheritDoc} */
	@Override
	public void rmAssociatedElement(IMappingEntry entry) {
		associatedEntries.remove(entry);
	}

	/** {@inheritDoc} */
	@Override
	public Collection<IMappingEntry> getAssociatedElement() {
		return associatedEntries.values();
	}

	/** {@inheritDoc} */
	@Override
	public <T extends IMappingEntry> T getAssociatedElement(Class<T> type) {
		return associatedEntries.getInstance(type);
	}

	/** {@inheritDoc} */
	@Override
	public abstract TaskMappingEntry clone();
}
