/*--------------------------------------------------------------------------+
$Id: codetemplates.xml 1 2011-01-01 00:00:01Z hoelzl $
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.constraint;

import org.eclipse.emf.ecore.impl.EObjectImpl;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IDeployableComponentAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IRequestAdapter;
import org.fortiss.af3.exploration.model.ExplorationConstraint;

/**
 * DSE-internal constraint to define the bounds for the number of replica of an
 * {@link IDeployableComponentAdapter}.
 * NOTE: This constraint is only a temporary solution until a solution is found which allows to use
 * the EMM for this task.
 * 
 * @author diewald
 * @author $Author: hoelzl $
 * @version $Rev: 18709 $
 * @ConQAT.Rating RED Hash: A75683844D4FCD4BA58DA4CE35A68C5A
 */
@SuppressWarnings("javadoc")
public class InternalReplicationConstraint extends EObjectImpl implements
		ExplorationConstraint<Boolean> {

	private int id = 0;

	private String name = "";

	private String comment = "";

	private boolean isSoft = false;

	private IRequestAdapter<?> replicationObject;

	private int min = 1;

	private int max = 1;

	public InternalReplicationConstraint(IRequestAdapter<?> replicationObject, int min, int max) {
		this.replicationObject = replicationObject;
		this.min = min;
		this.max = max;
	}

	public InternalReplicationConstraint(InternalReplicationConstraint other) {
		this.replicationObject = other.replicationObject;
		this.min = other.min;
		this.max = other.max;
		this.name = other.name;
		this.comment = other.comment;
		this.isSoft = other.isSoft;
	}

	/** {@inheritDoc} */
	@Override
	public String getComment() {
		return comment;
	}

	/** {@inheritDoc} */
	@Override
	public void setComment(String value) {
		comment = value;
	}

	/** {@inheritDoc} */
	@Override
	public String getName() {
		return name;
	}

	/** {@inheritDoc} */
	@Override
	public void setName(String value) {
		name = value;
	}

	/** {@inheritDoc} */
	@Override
	public int getId() {
		return id;
	}

	/** {@inheritDoc} */
	@Override
	public void setId(int value) {
		id = value;
	}

	/** {@inheritDoc} */
	@Override
	public boolean isIsSoft() {
		return isSoft;
	}

	/** {@inheritDoc} */
	@Override
	public void setIsSoft(boolean value) {
		isSoft = value;
	}

	/** {@inheritDoc} */
	@Override
	public boolean isImplicit() {
		return true;
	}

	/** {@inheritDoc} */
	@Override
	public void setImplicit(boolean value) {
		// NOP.
	}

	public int getMin() {
		return min;
	}

	public void setMin(int value) {
		min = value;
	}

	public int getMax() {
		return max;
	}

	public void setMax(int value) {
		max = value;
	}

	public IRequestAdapter<?> getReplicationObject() {
		return replicationObject;
	}

	public void setReplicationObject(IRequestAdapter<?> value) {
		replicationObject = value;
	}

	/** {@inheritDoc} */
	@Override
	public Object clone() {
		return new InternalReplicationConstraint(this);
	}
}
