/*--------------------------------------------------------------------------+
$Id: codetemplates.xml 1 2011-01-01 00:00:01Z hoelzl $
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.constraint;

import java.util.Collection;

import org.eclipse.emf.ecore.impl.EObjectImpl;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IDeployableComponentAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IRequestAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.mapping.IMappingEntry;
import org.fortiss.af3.exploration.model.ExplorationConstraint;
import org.fortiss.af3.platform.model.IPlatformResource;

/**
 * DSE-internal constraint to define which {@link IMappingEntry}s of a
 * {@link IDeployableComponentAdapter} needs to be spatially separated.
 * NOTE: This constraint is only a temporary solution until a solution is found which allows to use
 * the EMM for this task.
 * 
 * @author diewald
 * @author $Author: hoelzl $
 * @version $Rev: 18709 $
 * @ConQAT.Rating RED Hash: 61AC18C4A081EA4D9310B99CE5D81846
 */
@SuppressWarnings("javadoc")
public class InternalSeparationConstraint extends EObjectImpl implements
		ExplorationConstraint<Boolean>, Cloneable {

	private int id = 0;

	private String name = "";

	private String comment = "";

	private boolean isSoft = false;

	private Collection<? extends IRequestAdapter<?>> requesterSet;

	private Class<? extends IPlatformResource> separationLayer;

	public InternalSeparationConstraint(Collection<? extends IRequestAdapter<?>> requesterSet,
			Class<? extends IPlatformResource> separationLayer) {
		this.requesterSet = requesterSet;
		this.separationLayer = separationLayer;
	}

	public InternalSeparationConstraint(InternalSeparationConstraint other) {
		this.requesterSet = other.requesterSet;
		this.separationLayer = other.separationLayer;
		this.name = other.name;
		this.comment = other.comment;
		this.isSoft = other.isSoft;
	}

	/** {@inheritDoc} */
	@Override
	public String getComment() {
		return comment;
	}

	/** {@inheritDoc} */
	@Override
	public void setComment(String value) {
		comment = value;
	}

	/** {@inheritDoc} */
	@Override
	public String getName() {
		return name;
	}

	/** {@inheritDoc} */
	@Override
	public void setName(String value) {
		name = value;
	}

	/** {@inheritDoc} */
	@Override
	public int getId() {
		return id;
	}

	/** {@inheritDoc} */
	@Override
	public void setId(int value) {
		id = value;
	}

	/** {@inheritDoc} */
	@Override
	public boolean isIsSoft() {
		return isSoft;
	}

	/** {@inheritDoc} */
	@Override
	public void setIsSoft(boolean value) {
		isSoft = value;
	}

	/** {@inheritDoc} */
	@Override
	public boolean isImplicit() {
		return true;
	}

	/** {@inheritDoc} */
	@Override
	public void setImplicit(boolean value) {
		// NOP.
	}

	public Collection<? extends IRequestAdapter<?>> getRequesterSet() {
		return requesterSet;
	}

	public void setRequesterSet(Collection<? extends IRequestAdapter<?>> value) {
		requesterSet = value;
	}

	public Class<? extends IPlatformResource> getSeparationLayer() {
		return separationLayer;
	}

	public void setSeparationLayer(Class<? extends IPlatformResource> value) {
		separationLayer = value;
	}

	/** {@inheritDoc} */
	@Override
	public Object clone() {
		return new InternalSeparationConstraint(this);
	}
}
