/*--------------------------------------------------------------------------+
$Id: FailSilentTaskMappingEntry.java 3032 2016-06-16 18:13:12Z diewald $
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IDeployableComponentAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;

/**
 * {@link TaskMappingEntry} used for encoding redundant {@link FailSilentTaskMappingEncoding}s (with
 * fail-silent assumption)
 * 
 * @author barner
 * @author $Author: diewald $
 * @version $Rev: 3032 $
 * @ConQAT.Rating YELLOW Hash: 8BD1925CD7E8FAB425D80C4A640C3920
 */
public class FailSilentTaskMappingEntry extends InstantiatedTaskMappingEntry implements Cloneable {

	/** Constructs a mapping entry the represents the mapping to a given deployment target. */
	FailSilentTaskMappingEntry(IDeployableComponentAdapter<?> deployableComponent,
			IExecutionUnitAdapter<?> deploymentTarget) {
		super(deployableComponent, deploymentTarget);
	}

	/**
	 * Constructor that generates a {@link FailSilentTaskMappingEntry} of a
	 * {@link FailSilentTaskMappingEncoding} for each {@link AbstractTaskMappingEntry} of a
	 * {@link AbstractTaskMappingEncoding}.
	 */
	public FailSilentTaskMappingEntry(AbstractTaskMappingEntry abstractEntry) {
		super(abstractEntry);
	}

	/** {@inheritDoc} */
	@Override
	public FailSilentTaskMappingEntry clone() {
		return new FailSilentTaskMappingEntry(getDeployableComponent(), getTarget());
	}

	/** {@inheritDoc} */
	@Override
	public String toString() {
		return getTarget().getName();
	}
}
