/*--------------------------------------------------------------------------+
$Id: FaultDetectionVotingTaskMappingEntry.java 2419 2016-03-04 18:05:52Z diewald $
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IDeployableComponentAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;

/**
 * {@link TaskMappingEntry} used for encoding redundant
 * {@link FaultDetectionVotingTaskMappingEncoding}s what consider fault detection and voting.
 * 
 * @author barner
 * @author $Author: diewald $
 * @version $Rev: 2419 $
 * @ConQAT.Rating RED Hash: A3A454AB7D75346FD2EB5F86193905E6
 */
public class FaultDetectionVotingTaskMappingEntry extends InstantiatedTaskMappingEntry implements
		Cloneable {
	/** Fault detector ID */
	// TODO: Investigate use of IDs in reliability analysis.
	private int faultDetectorID;

	/**
	 * Constructs a mapping entry the represents the mapping to a given deployment target and which
	 * specifies the ID of the used fault detector
	 */
	public FaultDetectionVotingTaskMappingEntry(IDeployableComponentAdapter<?> deployableComponent,
			IExecutionUnitAdapter<?> target, int i) {
		super(deployableComponent, target);
		faultDetectorID = i;
	}

	/**
	 * Constructor that generates a {@link FailSilentTaskMappingEntry} of a
	 * {@link FailSilentTaskMappingEncoding} for each {@link AbstractTaskMappingEntry} of a
	 * {@link AbstractTaskMappingEncoding}.
	 */
	public FaultDetectionVotingTaskMappingEntry(AbstractTaskMappingEntry abstractEntry, int i) {
		super(abstractEntry);
		faultDetectorID = i;
	}

	/**
	 * Returns the ID of the used fault detector.
	 */
	public int getFaultDetectorID() {
		return faultDetectorID;
	}

	/** {@inheritDoc} */
	@Override
	public FaultDetectionVotingTaskMappingEntry clone() {
		return new FaultDetectionVotingTaskMappingEntry(getDeployableComponent(), getTarget(),
				faultDetectorID);
	}

	/** {@inheritDoc} */
	@Override
	public String toString() {
		String rval = getTarget().getName();

		rval = rval.concat("(" + faultDetectorID + ")");

		return rval;
	}
}
