/*--------------------------------------------------------------------------+
$Id: CompositeDecoder.java 3062 2016-06-20 17:59:20Z diewald $
|                                                                          |
| Copyright 2015 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode;

import java.util.Map;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.genotype.ComposableGenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.ComposablePhenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.CompositePhenotype;
import org.jgrapht.experimental.dag.DirectedAcyclicGraph.CycleFoundException;
import org.opt4j.core.Genotype;
import org.opt4j.core.genotype.CompositeGenotype;
import org.opt4j.core.problem.Decoder;

/**
 * {@link Decoder}s of {@link CompositeGenotype}s implement this interface. The {@link Decoder}s of
 * the {@link ComposableGenotype}s comprising the {@link CompositeGenotype} will be delegated by the
 * implementing class.
 * 
 * @author diewald
 * @author $Author: diewald $
 * @version $Rev: 3062 $
 * @ConQAT.Rating YELLOW Hash: 4AA2FD77827F0DB85FEB6A109FE6A126
 */
public interface CompositeDecoder<C extends CompositeGenotype<Class<? extends ComposableGenotype>, ? extends ComposableGenotype>, P extends CompositePhenotype<? extends Genotype>>
		extends Decoder<C, P> {

	/**
	 * Performs the merge of the sub-{@link Genotype}s of {#link V} to the {@link CompositeGenotype}
	 * {#link V}.
	 * 
	 * @param subPhenotypes
	 *            Map relating the sub-{@link Genotype}s with their decoding result (aka phenotype).
	 * @return The {@link CompositeGenotype} {#link P} or {@code null} if the merge fails.
	 */
	<CP extends ComposablePhenotype<? extends Genotype>> P mergeSubPhenotypes(
			Map<Class<CP>, CP> subPhenotypes);

	/**
	 * Adds a {@link ComposableDecoder} for the given {@link ComposablePhenotype}. The decoding
	 * will triggered automatically and user defined merge function will consider the resulting
	 * {@link ComposablePhenotype} of this {@link ComposableDecoder}.
	 * 
	 * @throws CycleFoundException
	 */
	// TODO: add doc for exception.
			<G extends ComposableGenotype, CP extends ComposablePhenotype<G>, D extends ComposableDecoder<G, CP>>
			void registerSubDecoder(Class<G> subGenotype, D subDecoder) throws CycleFoundException;

}
