/*--------------------------------------------------------------------------+
$Id: DecoderDependencyGraph.java 3747 2016-10-21 14:10:16Z diewald $
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.graph;

import java.util.Collection;
import java.util.Comparator;
import java.util.HashSet;
import java.util.Set;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.create.ComposableCreator;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode.ComposableDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.exception.MissingDependencyException;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.genotype.ComposableGenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.ComposablePhenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.decode.ConstraintGenerationDecoder;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.fortiss.af3.exploration.alg.graph.DependencyGraph;
import org.fortiss.af3.exploration.alg.service.IExplorationEncoding;
import org.opt4j.core.problem.Creator;
import org.opt4j.core.problem.Decoder;

/**
 * Defines the Execution order of {@link Decoder}s / {@link Creator}s etc.
 * 
 * @author diewald
 * @author $Author: diewald $
 * @version $Rev: 3747 $
 * @ConQAT.Rating YELLOW Hash: 15A8F3DB3C18984E47A4D35D8F259EE1
 */
public class DecoderDependencyGraph
		extends
		DependencyGraph<IExplorationEncoding, ComposableDecoder<ComposableGenotype, ComposablePhenotype<?>>, DecoderEdge<ComposableGenotype, ComposablePhenotype<?>>> {

	/** Set of {@link IExplorationEncoding} types that can be obtained from the SystemModelAdapter. */
	protected Set<Class<? extends IExplorationEncoding>> sysModelEncodings;

	/** Internal graph representation for the execution order of {@link Creator}s. */
	private HashSet<ComposableCreator<?>> registeredCreators = new HashSet<>();

	/** Constructor. */
	public DecoderDependencyGraph(SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter,
			Set<Class<? extends IExplorationEncoding>> pTypeCollection) {
		super(pTypeCollection, IExplorationEncoding.class);
		sysModelEncodings = systemModelAdapter.getRegisteredEncodingTypes();
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	protected void checkMissingDependencies() throws MissingDependencyException {
		for(ComposableDecoder<?, ?> dec : operatorExecOrder) {
			Collection<Class<?>> reqEncodings = dec.getRequiredTypes();
			for(Class<?> reqEnc : reqEncodings) {
				if(!dependencyDagGraph
						.containsVertex((Class<? extends IExplorationEncoding>)reqEnc) &&
						!registeredCreators.stream().anyMatch(
								c -> c.getCreatedEncoding().equals(reqEnc)) &&
						!sysModelEncodings.contains(reqEnc)) {
					throw new MissingDependencyException(dec,
							(Class<? extends IExplorationEncoding>)reqEnc, "");
				}
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	protected Comparator<ComposableDecoder<ComposableGenotype, ComposablePhenotype<?>>>
			getSubExecListComparator() {
		return new Comparator<ComposableDecoder<ComposableGenotype, ComposablePhenotype<?>>>() {
			@Override
			public int compare(ComposableDecoder<ComposableGenotype, ComposablePhenotype<?>> o1,
					ComposableDecoder<ComposableGenotype, ComposablePhenotype<?>> o2) {
				if(o1 instanceof ConstraintGenerationDecoder &&
						o2 instanceof ConstraintGenerationDecoder) {
					return 0;
				} else if(o1 instanceof ConstraintGenerationDecoder) {
					return 1;
				} else if(o2 instanceof ConstraintGenerationDecoder) {
					return -1;
				}
				return 0;
			}
		};
	}
}
