/*--------------------------------------------------------------------------+
$Id: codetemplates.xml 1 2011-01-01 00:00:01Z hoelzl $
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.encoding;

import java.util.Collection;

import org.fortiss.af3.exploration.alg.service.IExplorationEncoding;
import org.fortiss.af3.exploration.model.ExplorationConstraint;

import com.google.common.collect.Multimap;

/**
 * {@link ConstrainedEncoding}s are {@link IExplorationEncoding}s that may carry constraints which
 * limit the allowed operation performed on them.
 * 
 * @author diewald
 * @author $Author: hoelzl $
 * @version $Rev: 18709 $
 * @ConQAT.Rating RED Hash:
 */
public interface ConstrainedEncoding extends IExplorationEncoding {

	/**
	 * Returns the collection of {@link ExplorationConstraint}s that are of the given type.
	 * 
	 * @param type
	 *            of the {@link ExplorationConstraint}s.
	 * @return Collection of {@link ExplorationConstraint}s of the given type, or an empty
	 *         collection if none exist.
	 */
	<T extends ExplorationConstraint<?>> Collection<T> getConstraintsOf(Class<T> type);

	/**
	 * Returns the {@link Multimap} containing the complete set of constraints defined on a
	 * particular encoding which can be queried by their type.
	 */
	Multimap<Class<? extends ExplorationConstraint<?>>, ExplorationConstraint<?>> getConstraints();

	/**
	 * Adds a given collection of {@link ExplorationConstraint}s to the encoding which is identified
	 * by the given type. Existing {@link ExplorationConstraint}s identified by the same type are
	 * replaced.
	 * 
	 * @param type
	 *            to identify the {@link ExplorationConstraint}s.
	 * @param constraints
	 *            Collection of {@link ExplorationConstraint}s to be added to the encoding.
	 */
	<T extends ExplorationConstraint<?>> void replaceConstraints(Class<T> type,
			Collection<T> constraints);

	/**
	 * Replaces all existing constraint sets with identical keys with those present in the given
	 * map.
	 * 
	 * @param constraintMap
	 *            Replacement constraints.
	 */
	<T extends ExplorationConstraint<?>> void replaceConstraints(
			Multimap<Class<? extends T>, T> constraintMap);

	/**
	 * Adds the given {@code constraint} to the set of constraints that is identified by the given
	 * {@code type}.
	 * 
	 * @param type
	 *            Constraint set identifier.
	 * @param constraint
	 *            Constraint to add to possibly existing constraint set.
	 */
	<T extends ExplorationConstraint<?>> void putConstraint(Class<T> type, T constraint);

	/**
	 * Removes the given {@code constraint} from the set of constraints that is identified by the
	 * given {@code type}.
	 * 
	 * @param type
	 *            Constraint set identifier.
	 * @param constraint
	 *            Constraint to remove from the constraint set.
	 */
	<T extends ExplorationConstraint<?>> void removeConstraint(Class<T> type, T constraint);
}
