/*--------------------------------------------------------------------------+
$Id: CrossoverAbstractTaskMappingModule.java 3032 2016-06-16 18:13:12Z diewald $
|                                                                          |
| Copyright 2014 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.crossover;

import org.fortiss.af3.exploration.alg.dse.TaskMappingFactory;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;

/**
 * Crossover module selecting the {@link CrossoverRandManyAllocation} operator to be applied on
 * {@link AbstractTaskMappingEncoding}s.
 * 
 * @author huang
 * @author $Author: diewald $
 * @version $Rev: 3032 $
 * @ConQAT.Rating YELLOW Hash: AB046BE4BB327E6EA4F65A2D2E61C433
 */
public class CrossoverAbstractTaskMappingModule extends CrossoverOperatorModuleBase {
	/** Crossover rate */
	private double rate;

	/** References the {@link SystemModelAdapter} that handles the input models. */
	SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter;

	/** Constructor where execution model and mutation rate has to be specified */
	public CrossoverAbstractTaskMappingModule(
			SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter, double rate) {
		this.systemModelAdapter = systemModelAdapter;
		this.rate = rate;
	}

	/** {@inheritDoc} */
	@Override
	public void config() {
		super.config();

		addOperatorInstance(AbstractTaskMappingEncoding.class, TaskMappingFactory.getInstance()
				.createCrossoverRandManyAllocationOperator(systemModelAdapter, rate));
	}
}
