/*--------------------------------------------------------------------------+
$Id: CrossoverRandManyAllocation.java 3032 2016-06-16 18:13:12Z diewald $
|                                                                          |
| Copyright 2014 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.crossover;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IDeployableComponentAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.opt4j.operators.crossover.Pair;

/**
 * Applies the crossover operation on the allocations of {@link IDeployableComponentAdapter}s to
 * {@link IExecutionUnitAdapter}s. It requires two parent {@link TaskMappingEncoding}s whose
 * allocations are randomly swapped to generate two new children {@link TaskMappingEncoding}s.
 * 
 * @author huang
 * @author diewald
 * @author $Author: diewald $
 * @version $Rev: 3032 $
 * @ConQAT.Rating YELLOW Hash: 97966775B1ABA8E1DAE3305283299629
 */
public class CrossoverRandManyAllocation<S extends TaskMappingEntry, T extends TaskMappingEncoding<S>>
		extends CrossoverAllocationBase<S, T> {

	/** Constructor that allows to set the cross-over rate */
	public CrossoverRandManyAllocation(SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter,
			double rate) {
		super(systemModelAdapter, rate);
	}

	/** {@inheritDoc} */
	@Override
	public Pair<T> crossover(T encoding1, T encoding2) {
		@SuppressWarnings("unchecked") T newEncoding1 = (T)encoding1.clone();
		@SuppressWarnings("unchecked") T newEncoding2 = (T)encoding2.clone();

		for(IDeployableComponentAdapter<?> deployableComponent : encoding1.getDeployedComponents()) {
			if(rand.nextDouble() < rate) {
				crossoverAllocation(deployableComponent, newEncoding1, newEncoding2);
			}
		}

		// Check whether the a modification happened, if not enforce one.
		if(newEncoding1 == encoding1 || newEncoding1 == encoding2) {
			int randomIdx = encoding1.getDeployedComponents().toArray().length - 1;
			IDeployableComponentAdapter<?> randomlySelectedComponent =
					(IDeployableComponentAdapter<?>)encoding1.getDeployedComponents().toArray()[randomIdx];
			crossoverAllocation(randomlySelectedComponent, newEncoding1, newEncoding2);
		}

		return new Pair<T>(newEncoding1, newEncoding2);
	}
}
