/*--------------------------------------------------------------------------+
$Id: MutateOperatorModuleBase.java 2620 2016-04-29 11:56:14Z diewald $
|                                                                          |
| Copyright 2014 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.mutate;

import java.util.HashMap;
import java.util.Map;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.OperatorUtil;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.selection.IMultiSelectorOperatorModule;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.selection.IOperatorSelector;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.selection.RoundRobinOperatorSelector;
import org.opt4j.core.Genotype;
import org.opt4j.core.optimizer.Operator;
import org.opt4j.operators.mutate.BasicMutateModule;
import org.opt4j.operators.mutate.Mutate;

/**
 * Base class for mutate operator module that enables the registration of pre-constructed operators,
 * which thus can be constructed using a non-default constructor.
 * 
 * @author barner
 * @author $Author: diewald $
 * @version $Rev: 2620 $
 * @ConQAT.Rating YELLOW Hash: 646BE33B3011AFACCDADEFCDF6030C14
 */
public abstract class MutateOperatorModuleBase extends BasicMutateModule implements
		IMultiSelectorOperatorModule<Mutate<?>> {
	/**
	 * Contains the correspondences between the {@link Genotype}s and the {@link IOperatorSelector}
	 * selecting the {@link Operator}s used to modify the {@link Genotype}s.
	 */
	private Map<Class<? extends Genotype>, IOperatorSelector> selectorForGenotype =
			new HashMap<Class<? extends Genotype>, IOperatorSelector>();

	/** {@inheritDoc} */
	@Override
	public void
			addOperatorInstance(Class<? extends Genotype> genotypeType, Mutate<?> mutateOperator) {
		if(selectorForGenotype.get(genotypeType) == null) {
			selectorForGenotype.put(genotypeType, new RoundRobinOperatorSelector());
		}
		OperatorUtil.addOperatorInstance(genotypeType, mutateOperator,
				selectorForGenotype.get(genotypeType), binder(), getOperatorTypeLiteral());

	}

	/** {@inheritDoc} */
	@Override
	public IOperatorSelector getOperatorSelector(Class<? extends Genotype> genotypeType) {
		return selectorForGenotype.get(genotypeType);
	}
}
