/*--------------------------------------------------------------------------+
$Id: MutateChannelNumber.java 2686 2016-05-04 16:23:35Z diewald $
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.mutate.safetyarch;

import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.safetyfunctionarch.SafetyFunctionArchEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.safetyfunctionarch.SafetyFunctionArchEntry;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.opt4j.operators.mutate.Mutate;

/**
 * {@link Mutate} operator that varies the number of channels which realize a safety function.
 * 
 * @author diewald
 * @author $Author: diewald $
 * @version $Rev: 2686 $
 * @ConQAT.Rating YELLOW Hash: 5D1E5C8BEE5A641F2488ACAC50EA0AD7
 */
public class MutateChannelNumber<C> extends MutateSafetyArchBase<C> {

	/** Constructor. */
	protected MutateChannelNumber(SystemModelAdapter<C, ?, ?, ?, ?, ?> systemModelAdapter) {
		super(systemModelAdapter);
	}

	/** {@inheritDoc} */
	@Override
	public void mutate(SafetyFunctionArchEncoding<C> genotype, double p) {
		// Determine the safety functions that shall be manipulated.
		Collection<SafetyFunctionArchEntry<C>> selEntries = selectSafetyFunctions(genotype, p);

		for(SafetyFunctionArchEntry<C> sfEntry : selEntries) {
			int[] allowedChannels = sfEntry.getSafetyFunctionAdapter().getAllowedChannelNumbers();
			int selectedChannelIdx = sfEntry.getNumChannels();
			if(rand.nextBoolean()) {
				// Increase number of channels.
				selectedChannelIdx++;
				selectedChannelIdx =
						selectedChannelIdx < allowedChannels.length ? selectedChannelIdx : 0;
			} else {
				// Decrease number of channels.
				selectedChannelIdx--;
				selectedChannelIdx =
						selectedChannelIdx < 0 ? selectedChannelIdx : allowedChannels.length - 1;
			}
			sfEntry.setNumChannels(allowedChannels[selectedChannelIdx]);
		}
	}
}
