/*--------------------------------------------------------------------------+
$Id: MutateConnectedDiagnosisPresent.java 3382 2016-08-08 15:26:54Z diewald $
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.mutate.safetyarch;

import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.safetyfunctionarch.SafetyFunctionArchEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.safetyfunctionarch.SafetyFunctionArchEntry;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.opt4j.operators.mutate.Mutate;

/**
 * {@link Mutate} operator that alters the presence of a diagnostic unit of a safety function which
 * is connected to the output unit of a safety function.
 * 
 * @author diewald
 * @author $Author: diewald $
 * @version $Rev: 3382 $
 * @ConQAT.Rating YELLOW Hash: D579745244D294840600338F94EDD499
 */
public class MutateConnectedDiagnosisPresent<C> extends MutateSafetyArchBase<C> {

	/** Constructor. */
	protected MutateConnectedDiagnosisPresent(
			SystemModelAdapter<C, ?, ?, ?, ?, ?> systemModelAdapter) {
		super(systemModelAdapter);
	}

	/** {@inheritDoc} */
	@Override
	public void mutate(SafetyFunctionArchEncoding<C> genotype, double p) {
		// Determine the safety functions that shall be manipulated.
		Collection<SafetyFunctionArchEntry<C>> selEntries = selectSafetyFunctions(genotype, p);

		for(SafetyFunctionArchEntry<C> sfEntry : selEntries) {
			// Toggle the presence of a diagnostic unit for this safety function. This has only an
			// effect if the selected architecture supports additional diagnostic units.
			// 2/3 --> equal distribution of choices considering the connection to an output unit.
			boolean diagPresent = sfEntry.isDiagnosicUnitPresent();
			if(rand.nextInt(3) < 2) {
				sfEntry.setDiagnosicUnitPresent(!diagPresent);
			}

			if(diagPresent && rand.nextBoolean()) {
				sfEntry.setDiagnosicUnitConnected(!sfEntry.isDiagnosicUnitConnected());
			}
		}
	}
}
