/*--------------------------------------------------------------------------+
$Id: MutateSafetyFunctionArchitectureModule.java 2686 2016-05-04 16:23:35Z diewald $
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.mutate.safetyarch;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.safetyfunctionarch.SafetyFunctionArchEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.mutate.MutateOperatorModuleBase;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.opt4j.core.Genotype;
import org.opt4j.operators.mutate.Mutate;

import com.google.inject.Module;

/**
 * {@link Module} registering the {@link Mutate} operators within Opt4J that alter the
 * {@link SafetyFunctionArchEncoding} {@link Genotype}.
 * 
 * @author diewald
 * @author $Author: diewald $
 * @version $Rev: 2686 $
 * @ConQAT.Rating YELLOW Hash: D0C0552AD6F8171AED48FB05964ED74A
 */
public class MutateSafetyFunctionArchitectureModule extends MutateOperatorModuleBase {
	/** Representation of input system */
	private SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter;

	/** Constructor where the input system model has to be specified. */
	public MutateSafetyFunctionArchitectureModule(
			SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter) {
		this.systemModelAdapter = systemModelAdapter;
	}

	/** {@inheritDoc} */
	@SuppressWarnings({"rawtypes", "unchecked"})
	@Override
	public void config() {
		super.config();

		addOperatorInstance(SafetyFunctionArchEncoding.class, new MutateChannelNumber(
				systemModelAdapter));
		addOperatorInstance(SafetyFunctionArchEncoding.class, new MutateSafetyTriggerSignalNumber(
				systemModelAdapter));
		addOperatorInstance(SafetyFunctionArchEncoding.class, new MutateConnectedDiagnosisPresent(
				systemModelAdapter));
	}
}
