/*--------------------------------------------------------------------------+
$Id: MutateTaskInstanceModule.java 3032 2016-06-16 18:13:12Z diewald $
|                                                                          |
| Copyright 2015 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.mutate.taskinstantiation;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.InstantiatedTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.mutate.MutateOperatorModuleBase;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.mutate.taskmapping.MutateTaskMappingModule;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.opt4j.operators.mutate.Mutate;

import com.google.inject.Module;

/**
 * {@link Module} registering the {@link MutateTaskInstance} operator within Opt4J for
 * {@link InstantiatedTaskMappingEncoding}s.
 * <p>
 * <b>NOTE:</b> Currently, the {@link MutateTaskInstance} operator is registered within the
 * {@link MutateTaskMappingModule}, as Opt4J currently does not allow to add multiple {@link Mutate}
 * modules. To fix this issue, Opt4J must allow defining the mutation rate per genotype. Once this
 * is resolved, this module will be used.
 * 
 * @author diewald
 * @author $Author: diewald $
 * @version $Rev: 3032 $
 * @ConQAT.Rating YELLOW Hash: F90DC38B06D5B858F25B31906E684DDD
 */
public class MutateTaskInstanceModule extends MutateOperatorModuleBase {

	/** Representation of input system */
	private SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter;

	/** Constructor where the input system model has to be specified. */
	public MutateTaskInstanceModule(SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter) {
		this.systemModelAdapter = systemModelAdapter;
	}

	/** {@inheritDoc} */
	@Override
	public void config() {
		super.config();

		addOperatorInstance(InstantiatedTaskMappingEncoding.class, new MutateTaskInstance(
				systemModelAdapter));
	}
}
