/*--------------------------------------------------------------------------+
$Id: MutateRedundancyDecrement.java 2620 2016-04-29 11:56:14Z diewald $
|                                                                          |
| Copyright 2014 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.mutate.taskmapping;

import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IDeployableComponentAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;

/**
 * Mutate operator that decreases the redundancy of allocations.
 * 
 * @author huang
 * @author $Author: diewald $
 * @version $Rev: 2620 $
 * @ConQAT.Rating YELLOW Hash: 04A4E99964EE38F3954ECFD65A1FABC0
 */
public class MutateRedundancyDecrement<S extends TaskMappingEntry, T extends TaskMappingEncoding<S>>
		extends MutateRedundancy<S, T> {

	/** Constructs a new decrement redundancy mutation operator for a given exploration run */
	public MutateRedundancyDecrement(SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter) {
		super(systemModelAdapter);
	}

	/** {@inheritDoc} */
	@Override
	protected void mutate(T encoding, IDeployableComponentAdapter<?> deployableComponent,
			int minReplicatation, int maxReplication) {
		Collection<S> assignedResources = encoding.getMappingEntriesOf(deployableComponent);
		assert (assignedResources != null);

		// Decrease redundancy: keep at least the minimum number of required deployments
		if(assignedResources.size() - 1 >= minReplicatation) {
			assignedResources.remove(rand.nextInt(assignedResources.size()));
		}
	}
}
