/*--------------------------------------------------------------------------+
$Id: RandomOperatorSelector.java 3032 2016-06-16 18:13:12Z diewald $
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.selection;

import org.fortiss.af3.exploration.alg.dse.sysmodel.random.RandomExploration;
import org.opt4j.core.optimizer.Operator;

/**
 * In each selection round (see {@link OperatorSelectorBase}), selects are a random operator from
 * the set of registered operators.
 * 
 * @author barner
 * @author $Author: diewald $
 * @version $Rev: 3032 $
 * @ConQAT.Rating YELLOW Hash: 6CF50A0DF6BC7FAD0AE17F2E24915321
 */
public class RandomOperatorSelector extends OperatorSelectorBase {
	/** Random generator */
	private RandomExploration rand = RandomExploration.getInstance();

	/** Operator selected for the current round (selected randomly for each selection round). */
	private int currentOperator;

	/** {@inheritDoc} */
	@Override
	protected void registerInternal(Operator<?> operator) {
		currentOperator = rand.nextInt(operatorToIndexMap.size());
	}

	/** {@inheritDoc} */
	@Override
	protected boolean evaluateInternal(Operator<?> operator) {
		boolean rval = (operatorToIndexMap.get(operator) == currentOperator);
		return rval;
	}

	/** {@inheritDoc} */
	@Override
	protected void finalizeSelectionRoundInternal() {
		currentOperator = rand.nextInt(operatorToIndexMap.size());
	}

}
