/*--------------------------------------------------------------------------+
$Id: RoundRobinOperatorSelector.java 1249 2015-09-10 14:54:25Z diewald $
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.selection;

import org.opt4j.core.optimizer.Operator;

/**
 * Selection strategy that selects operators in a round-robin fashion (in the registration order of
 * the operators).
 * 
 * @author barner
 * @author $Author: diewald $
 * @version $Rev: 1249 $
 * @ConQAT.Rating YELLOW Hash: FA30BC49FBE6CB59BAD642DDEC3AD865
 */
public class RoundRobinOperatorSelector extends OperatorSelectorBase {
	/** Operator selected for the current round */
	private int currentOperator;

	/** {@inheritDoc} */
	@Override
	protected boolean evaluateInternal(Operator<?> operator) {
		boolean rval = (operatorToIndexMap.get(operator) == currentOperator);
		return rval;
	}

	/** {@inheritDoc} */
	@Override
	protected void finalizeSelectionRoundInternal() {
		currentOperator++;
		currentOperator %= operatorToIndexMap.size();
	}
}
