/*--------------------------------------------------------------------------+
$Id: FaultDetectionVotingAbstractTaskMappingDecoder.java 3964 2016-12-12 12:55:09Z diewald $
|                                                                          |
| Copyright 2015 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.instantiatemapping;

import static org.fortiss.tooling.common.util.LambdaUtils.filterSet;

import java.util.ArrayList;
import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskgraph.AbstractTaskGraphEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.FaultDetectionVotingTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.FaultDetectionVotingTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.exception.DecodingException;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.FlatPhenotypeMap;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IDeployableComponentAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.fortiss.af3.exploration.moea.model.DseSpecification;
import org.opt4j.core.problem.Decoder;

/**
 * {@link Decoder} for instantiating an {@link AbstractTaskMappingEncoding} to a
 * {@link FaultDetectionVotingTaskMappingEncoding}.
 * 
 * @author diewald
 * @author $Author: diewald $
 * @version $Rev: 3964 $
 * @ConQAT.Rating RED Hash: 5E7E9C6D48A981289747E9A9AB800CC2
 */
public class FaultDetectionVotingAbstractTaskMappingDecoder
		extends
		AbstractTaskMappingDecoder<FaultDetectionVotingTaskMappingEntry, FaultDetectionVotingTaskMappingEncoding> {

	/** Constructor. */
	public FaultDetectionVotingAbstractTaskMappingDecoder(
			SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter,
			DseSpecification explorationDefinition) {
		super(systemModelAdapter, explorationDefinition);
	}

	/** {@inheritDoc} */
	@Override
	public FaultDetectionVotingTaskMappingEncoding decodeComposable(
			AbstractTaskMappingEncoding genotype, FlatPhenotypeMap<Phenotype> phenotypeTypeMap) {
		assert (false);
		AbstractTaskGraphEncoding<?, ?> atgEnc =
				phenotypeTypeMap.getEncoding(AbstractTaskGraphEncoding.class);
		assert (atgEnc != null) : "The required abstract task graph encoding could not be found in the map of already decoded phenotypes.";
		return new FaultDetectionVotingTaskMappingEncoding(explorationDefinition,
				systemModelAdapter, atgEnc, genotype);
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @throws DecodingException
	 *             see {@link DecodingException}.
	 */
	@Override
	public FaultDetectionVotingTaskMappingEncoding updateComposable(
			AbstractTaskMappingEncoding inGenotype,
			FaultDetectionVotingTaskMappingEncoding outGenotype,
			FlatPhenotypeMap<Phenotype> decodedPhenotypes) throws DecodingException {
		outGenotype.replaceConstraints(inGenotype.getConstraints());

		for(IDeployableComponentAdapter<?> comp : inGenotype.getDeployedComponents()) {
			Collection<AbstractTaskMappingEntry> abstrEntries =
					inGenotype.getMappingEntriesOf(comp);
			Collection<FaultDetectionVotingTaskMappingEntry> instEntries;
			if(comp.isInstantiationRequired()) {
				instEntries =
						filterSet(outGenotype.getMappingEntries(), e -> e.getDeployableComponent()
								.getReplacedComponent() == comp);
			} else {
				instEntries = outGenotype.getMappingEntriesOf(comp);
			}

			// Missing Associations between instantiated and abstract entries indicate the need to
			// create or remove corresp. entries.
			for(AbstractTaskMappingEntry abstrEntry : abstrEntries) {
				if(abstrEntry.getAssociatedElement().isEmpty()) {
					outGenotype.createAndAddEntry(abstrEntry);
				}
			}

			Collection<FaultDetectionVotingTaskMappingEntry> rmEntries = new ArrayList<>();
			for(FaultDetectionVotingTaskMappingEntry instEntry : instEntries) {
				if(instEntry.getAssociatedElement().isEmpty()) {
					rmEntries.add(instEntry);
				}
			}
			rmEntries.forEach(e -> outGenotype.removeEntry(e));
		}

		// Remove all tasks that are present in the output encoding but not in the input encoding.
		Collection<IDeployableComponentAdapter<?>> rmTasks = new ArrayList<>();
		for(IDeployableComponentAdapter<?> comp : outGenotype.getDeployedComponents()) {
			IDeployableComponentAdapter<?> rmComponent = comp;
			if(comp.isInstantiation()) {
				comp = comp.getReplacedComponent();
			}
			if(!inGenotype.getDeployedComponents().contains(comp)) {
				rmTasks.add(rmComponent);
			}
		}
		rmTasks.forEach(e -> outGenotype.removeTask(e));

		for(FaultDetectionVotingTaskMappingEntry entry : outGenotype.getMappingEntries()) {
			entry.resetTarget(entry.getAssociatedElement(AbstractTaskMappingEntry.class)
					.getTarget());
		}

		return outGenotype;
	}
}
