/*--------------------------------------------------------------------------+
$Id: InstantiatedTaskMappingDecoderAcyclic.java 3841 2016-11-04 14:51:29Z diewald $
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.instantiatetaskgraph;

import java.util.ArrayList;
import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.safetyfunctionarch.SafetyFunctionArchEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskgraph.AbstractTaskGraphEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskgraph.InstantiatedAcyclicTaskGraphEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskgraph.InstantiatedTaskGraphEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.InstantiatedTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.InstantiatedTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode.ComposableDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode.ComposableDecoderBase;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.exception.DecodingException;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.FlatPhenotypeMap;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;

import com.google.inject.Inject;

/**
 * {@link ComposableDecoder} that takes an {@link InstantiatedTaskMappingEncoding} as input and
 * generates an {@link InstantiatedAcyclicTaskGraphEncoding} from it.
 * 
 * @author diewald
 * @author $Author: diewald $
 * @version $Rev: 3841 $
 * @ConQAT.Rating RED Hash: 0B02396AF05E1BC519AA16A848BBD291
 */
public class InstantiatedTaskMappingDecoderAcyclic<T extends InstantiatedTaskMappingEntry, C>
		extends
		ComposableDecoderBase<InstantiatedTaskMappingEncoding<T>, InstantiatedAcyclicTaskGraphEncoding<T, C>> {

	/** Representation of input system model */
	private SystemModelAdapter<C, ?, ?, ?, ?, ?> systemModelAdapter;

	/** Constructs a new decoder to generate {@link InstantiatedTaskGraphEncoding}. */
	@Inject
	public InstantiatedTaskMappingDecoderAcyclic(
			SystemModelAdapter<C, ?, ?, ?, ?, ?> systemModelAdapter) {
		this.systemModelAdapter = systemModelAdapter;
	}

	/** {@inheritDoc} */
	@Override
	public InstantiatedAcyclicTaskGraphEncoding<T, C> decodeComposable(
			InstantiatedTaskMappingEncoding<T> genotype,
			FlatPhenotypeMap<Phenotype> phenotypeTypeMap) {
		// TODO: automate the retrieval of phenotypes.
		@SuppressWarnings("unchecked") AbstractTaskGraphEncoding<InstantiatedTaskMappingEncoding<T>, C> atgEnc =
				phenotypeTypeMap.getEncoding(AbstractTaskGraphEncoding.class);
		assert (atgEnc != null) : "The required abstract task graph encoding could not be found in the map of already decoded phenotypes.";

		@SuppressWarnings("unchecked") InstantiatedTaskGraphEncoding<T, C> itgEnc =
				phenotypeTypeMap.getEncoding(InstantiatedTaskGraphEncoding.class);
		assert (itgEnc != null) : "The required abstract task graph encoding could not be found in the map of already decoded phenotypes.";

		InstantiatedAcyclicTaskGraphEncoding<T, C> iAtgEnc =
				new InstantiatedAcyclicTaskGraphEncoding<T, C>(systemModelAdapter,
						itgEnc.getTaskGraphs());

		return iAtgEnc;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @throws DecodingException
	 *             see {@link ComposableDecoder}.
	 */
	@Override
	public InstantiatedAcyclicTaskGraphEncoding<T, C> updateComposable(
			InstantiatedTaskMappingEncoding<T> inGenotype,
			InstantiatedAcyclicTaskGraphEncoding<T, C> outGenotype,
			FlatPhenotypeMap<Phenotype> decodedPhenotypes) throws DecodingException {
		// Not required: The InstantiatedTaskGraphEncoding is a Phenotype only.
		// TODO: Throw a decoding exception, instead.
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<?>> getRequiredTypes() {
		Collection<Class<?>> reqTypes = new ArrayList<>();
		reqTypes.add(InstantiatedTaskMappingEncoding.class);
		reqTypes.add(InstantiatedTaskGraphEncoding.class);

		return reqTypes;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<?>> getOptionalTypes() {
		Collection<Class<?>> optTypes = new ArrayList<>();
		optTypes.add(SafetyFunctionArchEncoding.class);
		return optTypes;
	}

	/** {@inheritDoc} */
	@SuppressWarnings("rawtypes")
	@Override
	public Class<InstantiatedAcyclicTaskGraphEncoding> getOutputType() {
		return InstantiatedAcyclicTaskGraphEncoding.class;
	}
}
