/*--------------------------------------------------------------------------+
$Id: SFEncodingDecoder.java 3478 2016-09-09 09:54:32Z diewald $
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.safetyfunction;

import java.util.ArrayList;
import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.safetyfunctionarch.SafetyFunctionArchEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode.ComposableDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode.ComposableDecoderBase;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.exception.DecodingException;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.ComposablePhenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.FlatPhenotypeMap;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.ArchExpSubDecoder;

/**
 * Provides a {@link SafetyFunctionArchEncoding} to the set of {@link ComposablePhenotype} types.
 * This {@link ComposableDecoder} is an identity operation.
 * 
 * @author diewald
 * @author $Author: diewald $
 * @version $Rev: 3478 $
 * @ConQAT.Rating YELLOW Hash: 35F8CADA2BDF8313D268D04DD4D3B4AE
 */
public class SFEncodingDecoder<C> extends
		ComposableDecoderBase<SafetyFunctionArchEncoding<C>, SafetyFunctionArchEncoding<C>>
		implements ArchExpSubDecoder {

	/**
	 * {@inheritDoc}
	 * 
	 * @throws DecodingException
	 *             see {@link ComposableDecoderBase}.
	 */
	@Override
	public SafetyFunctionArchEncoding<C> decodeComposable(SafetyFunctionArchEncoding<C> genotype,
			FlatPhenotypeMap<Phenotype> decodedPhenotypes) throws DecodingException {
		return genotype;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @throws DecodingException
	 *             see {@link ComposableDecoderBase}.
	 */
	@Override
	public SafetyFunctionArchEncoding<C>
			updateComposable(SafetyFunctionArchEncoding<C> inGenotype,
					SafetyFunctionArchEncoding<C> outGenotype,
					FlatPhenotypeMap<Phenotype> decodedPhenotypes) throws DecodingException {
		return inGenotype;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<?>> getRequiredTypes() {
		Collection<Class<?>> reqTypes = new ArrayList<>();
		reqTypes.add(SafetyFunctionArchEncoding.class);
		return reqTypes;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<?>> getOptionalTypes() {
		Collection<Class<?>> optTypes = new ArrayList<>();
		return optTypes;
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public Class<SafetyFunctionArchEncoding<C>> getOutputType() {
		Class<? super SafetyFunctionArchEncoding<C>> retType = SafetyFunctionArchEncoding.class;
		return (Class<SafetyFunctionArchEncoding<C>>)retType;
	}
}
