/*--------------------------------------------------------------------------+
$Id: SFEncodingDecoder.java 3382 2016-08-08 15:26:54Z diewald $
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.taskmapping;

import java.util.ArrayList;
import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode.ComposableDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode.ComposableDecoderBase;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.ComposablePhenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.FlatPhenotypeMap;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.ArchExpSubDecoder;

/**
 * Provides a {@link AbstractTaskMappingEncoding} to the set of {@link ComposablePhenotype} types.
 * This {@link ComposableDecoder} is an identity operation.
 * 
 * @author diewald
 * @author $Author: diewald $
 * @version $Rev: 3382 $
 * @ConQAT.Rating YELLOW Hash: 72358BBDF71DA7EA959AA8B918D0589D
 */
public class AbstractTaskMappingIdentityDecoder extends
		ComposableDecoderBase<AbstractTaskMappingEncoding, AbstractTaskMappingEncoding> implements
		ArchExpSubDecoder {

	/** {@inheritDoc} */
	@Override
	public AbstractTaskMappingEncoding decodeComposable(AbstractTaskMappingEncoding genotype,
			FlatPhenotypeMap<Phenotype> decodedPhenotypes) {
		return genotype;
	}

	/** {@inheritDoc} */
	@Override
	public AbstractTaskMappingEncoding updateComposable(AbstractTaskMappingEncoding inGenotype,
			AbstractTaskMappingEncoding outGenotype, FlatPhenotypeMap<Phenotype> decodedPhenotypes) {
		return inGenotype;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<?>> getRequiredTypes() {
		Collection<Class<?>> reqTypes = new ArrayList<>();
		reqTypes.add(AbstractTaskMappingEncoding.class);
		return reqTypes;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<?>> getOptionalTypes() {
		Collection<Class<?>> optTypes = new ArrayList<>();
		return optTypes;
	}

	/** {@inheritDoc} */
	@Override
	public Class<AbstractTaskMappingEncoding> getOutputType() {
		return AbstractTaskMappingEncoding.class;
	}
}
