/*--------------------------------------------------------------------------+
$Id: ExplorationTargetEvaluatorBase.java 3770 2016-10-25 13:50:28Z diewald $
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.evaluator;

import java.util.Collection;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.FlatPhenotypeMap;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.fortiss.af3.exploration.alg.service.IExplorationTargetEvaluator;
import org.fortiss.af3.exploration.model.ExplorationConstraint;
import org.fortiss.af3.exploration.model.ExplorationTarget;
import org.fortiss.tooling.kernel.introspection.IIntrospectionDetailsItem;
import org.fortiss.tooling.kernel.introspection.IIntrospectionItem;
import org.opt4j.core.DoubleValue;
import org.opt4j.core.Value;
import org.opt4j.core.problem.Evaluator;

import com.google.common.collect.ClassToInstanceMap;

/**
 * Base class for {@link Evaluator}s of {@link ExplorationConstraint}s..
 * 
 * @author barner
 * @author $Author: diewald $
 * @version $Rev: 3770 $
 * @ConQAT.Rating YELLOW Hash: D88780841EF1C966921AC012DE64F562
 */
public abstract class ExplorationTargetEvaluatorBase<P, RT, T extends ExplorationTarget<RT>>
		implements IExplorationTargetEvaluator<P, RT, T> {

	/** Representation of system model */
	protected SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter;

	/** Exploration goal evaluated by this evaluator */
	protected T optTarget;

	/** Creates an evaluator for a given exploration goal */
	protected ExplorationTargetEvaluatorBase(
			SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter, T optTarget) {
		this.systemModelAdapter = systemModelAdapter;
		this.optTarget = optTarget;
	}

	/**
	 * Wrapper method for the original evaluate function.
	 * 
	 * @param transformedModels
	 *            TODO
	 */
	public Value<?> evaluate(P phenotype, FlatPhenotypeMap<Phenotype> phenotypeTypeMap,
			ClassToInstanceMap<EObject> transformedModels) {
		if(phenotype == null) {
			return new DoubleValue(Double.MAX_VALUE);
		}
		// TODO #checking: Add an check here to ensure that the evaluators don't mess around with
		// the encodings.
		Value<?> evalResult = evaluateGoal(phenotype, phenotypeTypeMap, transformedModels);
		return evalResult;
	}

	/** {@inheritDoc} */
	@Override
	public String getIntrospectionLabel() {
		return getClass().getSimpleName();
	}

	// TODO: Add the introspection information.
	/** {@inheritDoc} */
	@Override
	public boolean showInIntrospectionNavigation() {
		return true;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<IIntrospectionItem> getIntrospectionItems() {
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public IIntrospectionDetailsItem getDetailsItem() {
		return null;
	}
}
