/*--------------------------------------------------------------------------+
$Id: ComponentMultiAllocationConstraintEvaluator.java 3730 2016-10-19 15:43:43Z diewald $
|                                                                          |
| Copyright 2014 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.evaluator.constraint;

import java.util.Collection;
import java.util.Collections;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.exploration.alg.dse.SolutionQuantification;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.InstantiatedTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.InstantiatedTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.FlatPhenotypeMap;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IDeployableComponentAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.fortiss.af3.exploration.alg.service.IExplorationTargetEvaluator;
import org.fortiss.af3.exploration.model.ExplorationTarget;
import org.fortiss.af3.exploration.moea.model.predefined.ComponentMultiAllocationConstraint;
import org.opt4j.core.DoubleValue;

import com.google.common.collect.ClassToInstanceMap;

/**
 * Deployment constraint which defines the allowed deployments of
 * {@link IDeployableComponentAdapter}s to {@link IExecutionUnitAdapter}s. All other allocations are
 * forbidden.
 * 
 * @author huang
 * @author $Author: diewald $
 * @version $Rev: 3730 $
 * @ConQAT.Rating RED Hash: DCFC61B150B0DE35C808242C1D743496
 */
// TODO: Register this evaluator for AstractTaskMappings.
public class ComponentMultiAllocationConstraintEvaluator<S extends InstantiatedTaskMappingEntry, T extends InstantiatedTaskMappingEncoding<S>>
		extends ConstraintEvaluatorBase<T, Boolean, ComponentMultiAllocationConstraint> {

	/** Constructor. */
	public ComponentMultiAllocationConstraintEvaluator(
			SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter,
			ComponentMultiAllocationConstraint optGoal) {
		super(systemModelAdapter, optGoal);
	}

	/** {@inheritDoc} */
	@Override
	public DoubleValue evaluateGoal(T phenotype, FlatPhenotypeMap<Phenotype> phenotypeTypeMap,
			ClassToInstanceMap<EObject> transformedModels) {
		// Already considered in the operators.
		return new DoubleValue(0.0);
	}

	/** {@inheritDoc} */
	@Override
	public boolean validateGoalSpecification() {
		if(optTarget.getComponent() == null || optTarget.getExecutionUnits().isEmpty()) {
			return false;
		}
		return true;
	}

	/** {@inheritDoc} */
	@Override
	public boolean validateInternal(ExplorationTarget<?> expTarget, T solution,
			SolutionQuantification value) {
		// Already considered in the operators.
		// TODO: Just to be safe: add a real check.
		return true;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<?>> getRequiredInputTypes() {
		return Collections.emptySet();
	}

	/** {@inheritDoc} */
	@Override
	public Class<?> getPrimaryInputType() {
		return InstantiatedTaskMappingEncoding.class;
	}

	/** {@inheritDoc} */
	@Override
	public IExplorationTargetEvaluator<T, Boolean, ComponentMultiAllocationConstraint>
			createEvaluator(SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter,
					ComponentMultiAllocationConstraint explorationTarget) {
		return new ComponentMultiAllocationConstraintEvaluator<>(systemModelAdapter,
				explorationTarget);
	}
}
