/*--------------------------------------------------------------------------+
$Id: IConstraintEvaluator.java 3404 2016-08-26 15:01:08Z diewald $
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.evaluator.constraint;

import org.fortiss.af3.exploration.alg.dse.SolutionQuantification;
import org.fortiss.af3.exploration.model.ExplorationConstraint;
import org.fortiss.af3.exploration.model.ExplorationTarget;
import org.opt4j.core.problem.Evaluator;

/**
 * Interface for {@link Evaluator}s of constraints. Since constraints are hard requirements to
 * possible solutions a problem, a validate method is required to ensure that a feasible solution
 * set is provided bys the DSE, i.e. no constraints are violated by these solutions.
 * 
 * @author diewald
 * @author $Author: diewald $
 * @version $Rev: 3404 $
 * @ConQAT.Rating YELLOW Hash: 78D9D28E5DDB7FE8AC17A6CC1E19F364
 */
public interface IConstraintEvaluator<P, RT, T extends ExplorationConstraint<RT>> {

	/**
	 * The {@code validate} is applied to phenotypes in order to check whether proposed solutions
	 * are contained within the feasible set.
	 */
	public <M> boolean validate(ExplorationTarget<?> expTarget, M solution, Class<M> solutionType,
			SolutionQuantification value);
}
