/*--------------------------------------------------------------------------+
$Id: ITransformationStrategy.java 3456 2016-09-05 14:53:35Z diewald $
|                                                                          |
| Copyright 2015 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.modeltransformation;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.exploration.alg.exception.TransformationModuleException;
import org.fortiss.af3.project.model.FileProject;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;

/**
 * Interface that defines common operation to trigger a the generation of a model based on
 * implementation specific input models.
 * 
 * @author diewald
 * @author $Author: diewald $
 * @version $Rev: 3456 $
 * @ConQAT.Rating RED Hash: BFA97B7D2A97E5D1579C5D75FFA06119
 */
public interface ITransformationStrategy<T extends EObject> {

	/**
	 * Sets the {@code modelContext} that is used by {@link ITransformationStrategy}s if any model
	 * (transformation result) shall be added to the parent project.
	 * 
	 * @param modelContext
	 *            context of the parent's project.
	 */
	void setModelContext(ITopLevelElement modelContext);

	/**
	 * Sets the name of the resulting model. It is propagated to any sub
	 * {@link ITransformationStrategy} such that associated models can be identified by their names
	 * if implemented.
	 * 
	 * @param transformedModelName
	 *            Name to be set for the top-most requested model.
	 */
	void setModelName(String transformedModelName);

	/**
	 * Sets a flag to enable the {@link ITransformationStrategy} to detect whether it has been
	 * launched as the "mother" strategy or a sub-strategy.
	 */
	public void setLaunchedAsSubModule(boolean isSubModule);

	/**
	 * Adds the model resulting from the transformation to the given project that has the given
	 * context.
	 * 
	 * @param fp
	 *            The {@link FileProject} to which the transformed model shall be added.
	 * @param modelContext
	 *            The model context of the given project.
	 * @throws TransformationModuleException
	 *             if the adding operation fails (the enclosed exception contains the details).
	 */
	// TODO: remove the model context parameter, or the setModelContext method above.
	void addModelToProject(FileProject fp, ITopLevelElement modelContext)
			throws TransformationModuleException;
}
