/*--------------------------------------------------------------------------+
$Id: codetemplates.xml 1 2011-01-01 00:00:01Z hoelzl $
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.modeltransformation.base;

import static org.fortiss.tooling.kernel.utils.EcoreUtils.getChildrenWithType;

import java.util.Collection;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.exploration.alg.dse.modeltransformation.element.IElementTransformationModule;
import org.fortiss.af3.exploration.alg.exception.TransformationModuleException;
import org.fortiss.af3.exploration.alg.guava.SubClassToInstanceMap;
import org.fortiss.af3.exploration.alg.service.ExplorationEncodingMap;
import org.fortiss.af3.exploration.alg.service.IExplorationEncoding;
import org.fortiss.af3.exploration.alg.service.IExplorationTransformationService.TransformationContext;

/**
 * 
 * @author diewald
 * @author $Author: hoelzl $
 * @version $Rev: 18709 $
 * @ConQAT.Rating RED Hash:
 */
public abstract class ElementTransformationModuleBase<T extends EObject, O extends EObject> extends
		TransformationModuleBase<T> implements IElementTransformationModule<T, O> {

	/** {@inheritDoc} */
	@Override
	public <N extends EObject, S extends EObject> T transform(TransformationState state,
			TransformationContext context, SubClassToInstanceMap<S> baseModels,
			ExplorationEncodingMap<IExplorationEncoding> explorationEncodings,
			SubClassToInstanceMap<N> transformedModels, EObject outputModel)
			throws TransformationModuleException {
		if(outputModel == null) {
			throw new TransformationModuleException(this,
					"No output model has been passed. It is required to store the transformation results.");
		}

		// TODO: find a way to directly use the generic here.
		@SuppressWarnings("unchecked") O tempCastedOutputModel = (O)outputModel;
		Collection<T> elements = getChildrenWithType(outputModel, getElementClass());
		for(T currElement : elements) {
			transformElement(currElement, baseModels, explorationEncodings, transformedModels,
					tempCastedOutputModel);
		}

		@SuppressWarnings("unchecked") T castedOutputModel = (T)outputModel;
		return castedOutputModel;
	}
}
