/*--------------------------------------------------------------------------+
$Id: M2MCopier.java 3382 2016-08-08 15:26:54Z diewald $
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.modeltransformation.element;

import static org.fortiss.tooling.kernel.utils.EcoreUtils.addAll;

import java.util.LinkedHashMap;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.util.EcoreUtil.Copier;

/**
 * {@link Copier} that is actually a copy of the "HackedCopied" from the EcoreUtils, but is public.
 * It allows to access the correspondence map from original to copied elements (A {@link Copier} is
 * a {@link LinkedHashMap} .
 * 
 * @author diewald
 * @author $Author: diewald $
 * @version $Rev: 3382 $
 * @ConQAT.Rating RED Hash: C02E7A05F42DC56E95E6E04D77E47631
 */
public class M2MCopier extends Copier {

	// TODO: Instead of duplicating the HackedCopier code here, the Hacked copier should be made
	// public and the M2MCpoier should inherit from this one.
	/** {@inheritDoc} */
	@Override
	protected void copyContainment(EReference eReference, EObject eObject, EObject copyEObject) {
		if(eObject.eIsSet(eReference)) {
			if(eReference.isMany()) {
				@SuppressWarnings("unchecked") List<EObject> source =
						(List<EObject>)eObject.eGet(eReference);
				@SuppressWarnings("unchecked") List<EObject> target =
						(List<EObject>)copyEObject.eGet(getTarget(eReference));
				if(source.isEmpty()) {
					target.clear();
				} else {
					addAll(target, copyAll(source));
				}
			} else {
				EObject childEObject = (EObject)eObject.eGet(eReference);
				copyEObject.eSet(getTarget(eReference), childEObject == null ? null
						: copy(childEObject));
			}
		}
	}
}
