/*--------------------------------------------------------------------------+
$Id: FailSilentExecModelFactory.java 3730 2016-10-19 15:43:43Z diewald $
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.sysmodel;

import org.conqat.lib.commons.reflect.TypeConversionException;
import org.fortiss.af3.exploration.alg.dse.TaskMappingFactory;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.create.taskmapping.FailSilentTaskMappingCreator;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.create.taskmapping.TaskMappingCreator;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.FailSilentTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.FailSilentTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.graph.DecoderDependencyGraph;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.crossover.CrossoverRandManyAllocation;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.crossover.CrossoverSingleAllocation;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.mutate.taskmapping.MutateAllocation;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.mutate.taskmapping.MutateRedundancyDecrement;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.mutate.taskmapping.MutateRedundancyIncrement;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.EvaluatorWrapper;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.StrictTTDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.comm.MessageDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.instantiatemapping.AbstractTaskMappingDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.instantiatemapping.FailSilentAbstractTaskMappingDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.instantiatetaskgraph.InstantiatedTaskMappingDecoderAcyclic;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.solution.StrictTTSchedule;
import org.fortiss.af3.exploration.alg.dse.evaluator.constraint.ComponentMultiAllocationConstraintEvaluator;
import org.fortiss.af3.exploration.alg.dse.evaluator.constraint.ComponentMultiDislocationConstraintEvaluator;
import org.fortiss.af3.exploration.alg.dse.evaluator.constraint.DeadlineConstraintEvaluator;
import org.fortiss.af3.exploration.alg.dse.evaluator.constraint.PeriodConstraintEvaluator;
import org.fortiss.af3.exploration.alg.dse.evaluator.constraint.SafetyIntegrityLevelConstraintEvaluator;
import org.fortiss.af3.exploration.alg.dse.evaluator.objective.FailSilentReliabilityEvaluator;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.sched.proc.EDFSchedulerBase;
import org.fortiss.af3.exploration.alg.dse.sysmodel.sched.proc.FailSilentEDFScheduler;
import org.fortiss.af3.exploration.model.ExplorationSpecification;
import org.fortiss.af3.exploration.model.ExplorationTarget;
import org.fortiss.af3.exploration.model.time.DeadlineConstraint;
import org.fortiss.af3.exploration.model.time.PeriodConstraint;
import org.fortiss.af3.exploration.moea.model.DseSpecification;
import org.fortiss.af3.exploration.moea.model.predefined.ComponentMultiAllocationConstraint;
import org.fortiss.af3.exploration.moea.model.predefined.ComponentMultiDislocationConstraint;
import org.fortiss.af3.exploration.moea.model.predefined.FailureMinObjective;
import org.fortiss.af3.exploration.moea.model.predefined.SafetyIntegrityLevelConstraint;
import org.fortiss.af3.safety.model.SIL;

import com.google.inject.TypeLiteral;

/**
 * This class implements the creation of the EavaluatorWrapper for the fail silent execution model.
 * 
 * @author diewald
 * @author $Author: diewald $
 * @version $Rev: 3730 $
 * @ConQAT.Rating RED Hash: C1870E496AFD69E7768A422E5585FCA8
 */
public class FailSilentExecModelFactory extends
		TaskMappingFactory<FailSilentTaskMappingEntry, FailSilentTaskMappingEncoding> {

	/** The instance (singleton) of the EvaluatorWrapper factory. */
	public static TaskMappingFactory<?, ?> getInstance() {
		if(instance == null) {
			instance = new FailSilentExecModelFactory();
		}
		return instance;
	}

	/*
	 * Construct Creators for the task mapping
	 */

	/** {@inheritDoc} */
	@Override
	public TypeLiteral<FailSilentTaskMappingEncoding> createGenotypeTypeLiteral() {
		return new TypeLiteral<FailSilentTaskMappingEncoding>() {
			// Anonymous.
		};
	}

	/** Creates a TaskMappingCreator that matches the given execution model. */
	@Override
	public TaskMappingCreator<FailSilentTaskMappingEntry, FailSilentTaskMappingEncoding>
			createTaskMappingCreator(DseSpecification dse,
					SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter,
					DecoderDependencyGraph execDepGraph) {
		return new FailSilentTaskMappingCreator(dse, systemModelAdapter, execDepGraph);
	}

	// /** Creates a TaskMappingCreator that matches the given execution model. */
	// @Override
	// public TaskMappingCreator<FailSilentTaskMappingEntry, FailSilentTaskMappingEncoding>
	// createTaskMappingCreator(DseSpecification dse,
	// SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter,
	// AbstractTaskMappingEncoding abstractEnc) {
	// return new FailSilentTaskMappingCreator(dse, systemModelAdapter, abstractEnc);
	// }

	/*
	 * Construct execution model specific operators
	 */

	/** {@inheritDoc} */
	@Override
	public MutateAllocation<?, ?> createMutateAllocationOperator(
			SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter) {
		return new MutateAllocation<AbstractTaskMappingEntry, AbstractTaskMappingEncoding>(
				systemModelAdapter);
	}

	/** {@inheritDoc} */
	@Override
	public MutateRedundancyDecrement<?, ?> createMutateRedundancyDecrementOperator(
			SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter) {
		return new MutateRedundancyDecrement<AbstractTaskMappingEntry, AbstractTaskMappingEncoding>(
				systemModelAdapter);
	}

	/** {@inheritDoc} */
	@Override
	public MutateRedundancyIncrement<?, ?> createMutateRedundancyIncrementOperator(
			SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter) {
		return new MutateRedundancyIncrement<AbstractTaskMappingEntry, AbstractTaskMappingEncoding>(
				systemModelAdapter);
	}

	/** {@inheritDoc} */
	@Override
	public CrossoverRandManyAllocation<?, ?> createCrossoverRandManyAllocationOperator(
			SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter, double rate) {
		return new CrossoverRandManyAllocation<AbstractTaskMappingEntry, AbstractTaskMappingEncoding>(
				systemModelAdapter, rate);
	}

	/** {@inheritDoc} */
	@Override
	public CrossoverSingleAllocation<?, ?> createCrossoverSingleAllocationOperator(
			SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter, double rate) {
		return new CrossoverSingleAllocation<AbstractTaskMappingEntry, AbstractTaskMappingEncoding>(
				systemModelAdapter, rate);
	}

	/*
	 * Construct Decoders for the genotypes, aka task mappings.
	 */

	/** {@inheritDoc} */
	@Override
	public AbstractTaskMappingDecoder<FailSilentTaskMappingEntry, FailSilentTaskMappingEncoding>
			createAbstractTaskMappingDecoder(
					SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter,
					DseSpecification explorationDefinition) {
		return new FailSilentAbstractTaskMappingDecoder(systemModelAdapter, explorationDefinition);
	}

	/** {@inheritDoc} */
	@SuppressWarnings({"rawtypes", "unchecked"})
	@Override
	// TODO: identify why a warning is generated here.
	public
			InstantiatedTaskMappingDecoderAcyclic<FailSilentTaskMappingEntry, FailSilentTaskMappingEncoding>
			createAbstractToAcyclicTaskGraphDecoder(
					SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter) {
		return new InstantiatedTaskMappingDecoderAcyclic(systemModelAdapter);
	}

	/** {@inheritDoc} */
	@Override
	public MessageDecoder<FailSilentTaskMappingEntry, FailSilentTaskMappingEncoding>
			createMessageDecoder(SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter) {
		return new MessageDecoder<FailSilentTaskMappingEntry, FailSilentTaskMappingEncoding>(
				systemModelAdapter);
	}

	/** {@inheritDoc} */
	@Override
	public StrictTTDecoder<FailSilentTaskMappingEntry, FailSilentTaskMappingEncoding>
			createStrictTTDecoder(SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter,
					DseSpecification explorationDefinition) {
		return new StrictTTDecoder<FailSilentTaskMappingEntry, FailSilentTaskMappingEncoding>(
				systemModelAdapter, explorationDefinition);
	}

	/**
	 * Creates an basic EDF scheduler that is used to decode task mappings into schedules for the
	 * given execution model
	 */
	@Override
	public EDFSchedulerBase<?, ?> createEDFScheduler(
			SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter,
			ExplorationSpecification expSpec) {
		return new FailSilentEDFScheduler(systemModelAdapter, expSpec);
	}

	/*
	 * Construct Evaluators for Optimization Goals
	 */

	/**
	 * {@inheritDoc}
	 * 
	 * @throws TypeConversionException
	 */
	@Override
	@SuppressWarnings("unchecked")
	public <RT, ET extends ExplorationTarget<RT>> EvaluatorWrapper<?, Double, FailureMinObjective>
			createFailureMinObjectiveWrapper(ExplorationTarget<?> goal, int priority,
					SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter)
					throws TypeConversionException {
		return new EvaluatorWrapper<StrictTTSchedule<FailSilentTaskMappingEntry, FailSilentTaskMappingEncoding>, Double, FailureMinObjective>(
				(FailureMinObjective)goal,
				new FailSilentReliabilityEvaluator(systemModelAdapter, (FailureMinObjective)goal),
				priority,
				(Class<? extends StrictTTSchedule<FailSilentTaskMappingEntry, FailSilentTaskMappingEncoding>>)StrictTTSchedule.class);
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @throws TypeConversionException
	 */
	@SuppressWarnings("unchecked")
	@Override
	public <RT, ET extends ExplorationTarget<RT>> EvaluatorWrapper<?, Double, DeadlineConstraint>
			createDeadlineConstraintWrapper(ExplorationTarget<?> goal, int priority,
					SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter)
					throws TypeConversionException {
		return new EvaluatorWrapper<StrictTTSchedule<FailSilentTaskMappingEntry, FailSilentTaskMappingEncoding>, Double, DeadlineConstraint>(
				(DeadlineConstraint)goal,
				new DeadlineConstraintEvaluator<FailSilentTaskMappingEntry, FailSilentTaskMappingEncoding>(
						systemModelAdapter, (DeadlineConstraint)goal),
				priority,
				(Class<? extends StrictTTSchedule<FailSilentTaskMappingEntry, FailSilentTaskMappingEncoding>>)StrictTTSchedule.class);
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @throws TypeConversionException
	 */
	@SuppressWarnings("unchecked")
	@Override
	public <RT, ET extends ExplorationTarget<RT>> EvaluatorWrapper<?, Double, PeriodConstraint>
			createPeriodConstraintWrapper(ExplorationTarget<?> goal, int priority,
					SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter)
					throws TypeConversionException {
		return new EvaluatorWrapper<StrictTTSchedule<FailSilentTaskMappingEntry, FailSilentTaskMappingEncoding>, Double, PeriodConstraint>(
				(PeriodConstraint)goal,
				new PeriodConstraintEvaluator<FailSilentTaskMappingEntry, FailSilentTaskMappingEncoding>(
						systemModelAdapter, (PeriodConstraint)goal),
				priority,
				(Class<? extends StrictTTSchedule<FailSilentTaskMappingEntry, FailSilentTaskMappingEncoding>>)StrictTTSchedule.class);
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @throws TypeConversionException
	 */
	// TODO: adjust when the constraint is defined for mappings.
	@Override
	public <RT, ET extends ExplorationTarget<RT>>
			EvaluatorWrapper<?, Boolean, ComponentMultiAllocationConstraint>
			createComponentMultiAllocationConstraintWrapper(ExplorationTarget<?> goal,
					int priority, SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter)
					throws TypeConversionException {
		return new EvaluatorWrapper<FailSilentTaskMappingEncoding, Boolean, ComponentMultiAllocationConstraint>(
				(ComponentMultiAllocationConstraint)goal,
				new ComponentMultiAllocationConstraintEvaluator<FailSilentTaskMappingEntry, FailSilentTaskMappingEncoding>(
						systemModelAdapter, (ComponentMultiAllocationConstraint)goal), priority,
				FailSilentTaskMappingEncoding.class);
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @throws TypeConversionException
	 */
	// TODO: adjust when the constraint is defined for mappings.
	@Override
	public <RT, ET extends ExplorationTarget<RT>>
			EvaluatorWrapper<?, Boolean, ComponentMultiDislocationConstraint>
			createComponentMultiDislocationConstraintWrapper(ExplorationTarget<?> goal,
					int priority, SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter)
					throws TypeConversionException {
		return new EvaluatorWrapper<FailSilentTaskMappingEncoding, Boolean, ComponentMultiDislocationConstraint>(
				(ComponentMultiDislocationConstraint)goal,
				new ComponentMultiDislocationConstraintEvaluator<FailSilentTaskMappingEntry, FailSilentTaskMappingEncoding>(
						systemModelAdapter, (ComponentMultiDislocationConstraint)goal), priority,
				FailSilentTaskMappingEncoding.class);
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @throws TypeConversionException
	 */
	@Override
	public <RT, ET extends ExplorationTarget<RT>>
			EvaluatorWrapper<?, SIL, SafetyIntegrityLevelConstraint>
			createSafetyIntegrityLevelConstraintWrapper(ExplorationTarget<?> goal, int priority,
					SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter)
					throws TypeConversionException {
		return new EvaluatorWrapper<FailSilentTaskMappingEncoding, SIL, SafetyIntegrityLevelConstraint>(
				(SafetyIntegrityLevelConstraint)goal,
				new SafetyIntegrityLevelConstraintEvaluator<FailSilentTaskMappingEntry, FailSilentTaskMappingEncoding>(
						systemModelAdapter, (SafetyIntegrityLevelConstraint)goal), priority,
				FailSilentTaskMappingEncoding.class);
	}
}
