/*--------------------------------------------------------------------------+
$Id: ISignalAdapter.java 3038 2016-06-17 13:26:28Z diewald $
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.sysmodel.arch;

import java.util.Collection;

import org.fortiss.af3.exploration.extension.IDseInputParameters.SignalType;

/**
 * Adapter between logical channels of the system meta-model used in the modeling
 * tool, and the DSE.
 * 
 * @author barner
 * @author $Author: diewald $
 * @version $Rev: 3038 $
 * @ConQAT.Rating YELLOW Hash: 84B4884F77B75CAB0E723336E9300C44
 */
public interface ISignalAdapter<T> extends IRequestAdapter<T> {

	/** Returns the source component of this {@link ISignalAdapter}. */
	public IDeployableComponentAdapter<?> getSource();

	/** Returns the collection of target components of this {@link ISignalAdapter}. */
	public Collection<IDeployableComponentAdapter<?>> getTargets();

	/**
	 * Returns the size of the message sent via this {@link ISignalAdapter} if it was deployed onto
	 * the given {@link ICommunicationResourceAdapter}.
	 */
	public long getSignalSize();

	/**
	 * Returns the {@link SignalType} of a message if it were constructed from this
	 * {@link ISignalAdapter}.
	 */
	public SignalType getSignalType();

	/**
	 * Returns the transmission time of a message sent via this {@link ISignalAdapter} if it was
	 * deployed onto the given {@link ITransmissionUnitAdapter}.
	 */
	public double getTransmissionTime(ITransmissionUnitAdapter<?> transmissionUnit);
}
