/*--------------------------------------------------------------------------+
$Id: InternalComponentParameters.java 3038 2016-06-17 13:26:28Z diewald $
|                                                                          |
| Copyright 2015 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.sysmodel.arch;

import java.util.HashMap;
import java.util.Map;

import org.fortiss.af3.component.model.Component;

/**
 * Abstracts the deployment-specific parameter values of {@link Component}s (or other model
 * elements) and stores them in a simple map format. Currently, WCET and energy consumption
 * parameters are supported.
 * 
 * @author diewald
 * @author $Author: diewald $
 * @version $Rev: 3038 $
 * @ConQAT.Rating YELLOW Hash: 7491AA925020A646DED070A427CBE719
 */
public class InternalComponentParameters {

	/** Contains the WCET for each {@link IExecutionUnitAdapter}. */
	private Map<IExecutionUnitAdapter<?>, Double> wcetMap =
			new HashMap<IExecutionUnitAdapter<?>, Double>();

	/** Contains the energy consumption in Joule for each {@link IExecutionUnitAdapter}. */
	private Map<IExecutionUnitAdapter<?>, Double> energyConsumptionMap =
			new HashMap<IExecutionUnitAdapter<?>, Double>();

	/**
	 * Returns a map that contains the WCET values of the {@link Component} subject to the
	 * {@link IExecutionUnitAdapter}.
	 */
	public Map<IExecutionUnitAdapter<?>, Double> getWcetMap() {
		return wcetMap;
	}

	/**
	 * Sets a map that contains the WCET values of the {@link Component} subject to the
	 * {@link IExecutionUnitAdapter}.
	 */
	public void setWcetMap(Map<IExecutionUnitAdapter<?>, Double> wcetMap) {
		this.wcetMap = wcetMap;
	}

	/**
	 * Returns a map that contains the energy consumption values of the {@link Component} subject to
	 * the {@link IExecutionUnitAdapter} in Joule.
	 */
	public Map<IExecutionUnitAdapter<?>, Double> getEnergyConsumptionMap() {
		return energyConsumptionMap;
	}

	/**
	 * Sets a map that contains the energy consumption values of the {@link Component} subject to
	 * the {@link IExecutionUnitAdapter}.
	 */
	public void setEnergyConsumptionMap(Map<IExecutionUnitAdapter<?>, Double> energyConsumptionMap) {
		this.energyConsumptionMap = energyConsumptionMap;
	}

}
