/*--------------------------------------------------------------------------+
$Id: AF3ExecutionUnitAdapter.java 2419 2016-03-04 18:05:52Z diewald $
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.sysmodel.arch.af3;

import static org.fortiss.tooling.kernel.utils.EcoreUtils.pickFirstInstanceOf;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IDeployableComponentAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.platform.model.ExecutionUnit;
import org.fortiss.af3.safety.model.annotation.SafeFailureFraction;

/**
 * Adapter between AF3 {@link ExecutionUnit} (=deployment target of
 * {@link IDeployableComponentAdapter}s) and the DSE.
 * 
 * @author barner
 * @author $Author: diewald $
 * @version $Rev: 2419 $
 * @ConQAT.Rating RED Hash: 10411F2D6EB02AA92451248E47C0E2A6
 */
public class AF3ExecutionUnitAdapter extends AF3ResourceAdapter<ExecutionUnit> implements
		IExecutionUnitAdapter<ExecutionUnit> {
	/** Execution unit speed annotation */
	private double executionUnitPower;

	/** Execution unit safe failure fraction annotation. */
	private SafeFailureFraction safeFailureFraction;

	/**
	 * Creates a {@link IExecutionUnitAdapter} for an {@link ExecutionUnit}.
	 * 
	 * @param executionUnit
	 *            execution unit model element.
	 * @param failureProb
	 *            failure probability of the given {@link ExecutionUnit}.
	 * @param power
	 *            power consumed by the {@link ExecutionUnit} if active.
	 * 
	 * @throws Exception
	 */
	public AF3ExecutionUnitAdapter(ExecutionUnit executionUnit, Double failureProb, Double power)
			throws Exception {
		super(executionUnit, failureProb);
		this.executionUnitPower = power;

		safeFailureFraction =
				pickFirstInstanceOf(SafeFailureFraction.class, executionUnit.getSpecifications());
	}

	/** {@inheritDoc} */
	@Override
	public double getPower(double duration) {
		return executionUnitPower * duration;
	}

	/** {@inheritDoc} */
	@Override
	public double getSafeFailureFraction() {
		return safeFailureFraction.getValue();
	}
}
