/*--------------------------------------------------------------------------+
$Id: AF3GatewayAdapter.java 3730 2016-10-19 15:43:43Z diewald $
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.sysmodel.arch.af3;

import static org.fortiss.tooling.base.utils.AnnotationUtils.getAnnotation;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IGatewayUnitAdapter;
import org.fortiss.af3.exploration.alg.exception.InvalidPlatformModelException;
import org.fortiss.af3.platform.model.ExecutionUnit;
import org.fortiss.af3.platform.model.GatewayUnit;
import org.fortiss.af3.platform.model.IVirtualizationPlatformArchitectureElement;
import org.fortiss.af3.platform.model.annotation.ResourceLink;

/**
 * Adapter between the AF3 {@link GatewayUnit} and the DSE.
 * 
 * @author diewald
 * @author $Author: diewald $
 * @version $Rev: 3730 $
 * @ConQAT.Rating RED Hash: 83E5DB861E9058EB370CB6B8A8D5CEF0
 */
public class AF3GatewayAdapter extends AF3ResourceAdapter<GatewayUnit> implements
		IGatewayUnitAdapter<GatewayUnit> {

	/** Power consumption to transfer a message of n bits via this {@link GatewayUnit}. */
	private Double power;

	/**
	 * Creates a {@link IExecutionUnitAdapter} for an {@link ExecutionUnit}.
	 * 
	 * @param gatewayUnit
	 *            {@link GatewayUnit} to create an adapter for.
	 * @param failureProb
	 *            failure probability of the {@link GatewayUnit}.
	 * 
	 * @throws Exception
	 *             if the {@link GatewayUnit} is part of the virtual platform (e.g., a driver) that
	 *             cannot be associated with any hardware element.
	 */
	public AF3GatewayAdapter(GatewayUnit gatewayUnit, Double failureProb, Double power)
			throws Exception {
		super(gatewayUnit, failureProb);
		this.power = power;

		if(gatewayUnit instanceof IVirtualizationPlatformArchitectureElement) {
			ResourceLink resLink = getAnnotation(gatewayUnit, ResourceLink.class);
			if(resLink == null) {
				throw new InvalidPlatformModelException(gatewayUnit,
						"No Resource Link annotation has been found: None is attached to this model element.");
			}

			if(resLink.getResources() != null && resLink.getResources().size() == 0) {
				throw new InvalidPlatformModelException(
						gatewayUnit,
						"One (or even more) gateway units must be referenced by an OnChipNetworkDriver in the Resource Link annotation. Each driver must be associated with at least one hardware element that is managed by the driver.");
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	public double getWCFT() {
		// TODO: add timing annotation for gateways
		// Currently, do nothing but returning 1.0s, since the timing annotation is missing for
		// Gateways
		return 0.000001;
	}

	/** {@inheritDoc} */
	@Override
	public double getPower(long messageSize) {
		// TODO: Add an appropriate bandwidth annotation for gateway units
		if(power == null) {
			return 1.0;
		}
		return power;
		// return messageSize * transmissionUnitBandwidth.getBandwidth_MBytesPerSecond() / 1e6;
	}
}
