/*--------------------------------------------------------------------------+
$Id: AF3MemoryUnitAdapter.java 3038 2016-06-17 13:26:28Z diewald $
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.sysmodel.arch.af3;

import static org.fortiss.tooling.kernel.utils.EcoreUtils.pickFirstInstanceOf;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IMemoryUnitAdapter;
import org.fortiss.af3.platform.model.MemoryUnit;
import org.fortiss.af3.platform.model.PlatformArchitecture;
import org.fortiss.af3.platform.model.annotation.MemorySize;

/**
 * {@link IMemoryUnitAdapter} for memory resources of an AF3 {@link PlatformArchitecture}.
 * 
 * @author diewald
 * @author $Author: diewald $
 * @version $Rev: 3038 $
 * @ConQAT.Rating YELLOW Hash: BBFE26C95D003DEE24AECDA3586F9C3E
 */
public class AF3MemoryUnitAdapter extends AF3ResourceAdapter<MemoryUnit> implements
		IMemoryUnitAdapter<MemoryUnit> {

	/** References the {@link MemorySize} annotation. */
	private MemorySize memorySizeSpec;

	/**
	 * Constructor.
	 * 
	 * @param modelElement
	 *            {@link MemoryUnit} from the target platform.
	 * @param failureProb
	 *            failure probability of the memory unit.
	 * 
	 * @throws Exception
	 *             if the memory size specification is missing.
	 */
	public AF3MemoryUnitAdapter(MemoryUnit modelElement, Double failureProb) throws Exception {
		super(modelElement, failureProb);

		memorySizeSpec = pickFirstInstanceOf(MemorySize.class, modelElement.getSpecifications());

		if(memorySizeSpec == null) {
			throw new Exception("The memory element " + modelElement +
					" does not have a MemorySize annotation.");
		}
	}

	/** {@inheritDoc} */
	@Override
	public long getSizeInBytes() {
		return memorySizeSpec.getSize();
	}

}
