/*--------------------------------------------------------------------------+
$Id: AF3ModelElementAdapter.java 3038 2016-06-17 13:26:28Z diewald $
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.sysmodel.arch.af3;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IModelElementAdapter;
import org.fortiss.af3.platform.model.ExecutionUnit;
import org.fortiss.tooling.base.model.element.IModelElement;
import org.fortiss.tooling.kernel.model.INamedElement;

/**
 * Base adapter for {@link IModelElementAdapter} based model elements.
 * 
 * @author barner
 * @author $Author: diewald $
 * @version $Rev: 3038 $
 * @ConQAT.Rating YELLOW Hash: 7E5558C5819EB922831EED51AD7A32A0
 */
public class AF3ModelElementAdapter<T extends IModelElement> implements IModelElementAdapter<T> {
	/** Reference to underlying model element */
	protected T modelElement;

	/**
	 * Creates a {@link IExecutionUnitAdapter} for an {@link ExecutionUnit}.
	 */
	public AF3ModelElementAdapter(T modelElement) {
		this.modelElement = modelElement;

	}

	/** {@inheritDoc} */
	@Override
	public String getName() {
		if(modelElement instanceof INamedElement) {
			return ((INamedElement)modelElement).getName() + "(id: " +
					((INamedElement)modelElement).getId() + ")";
		}
		return "<unnamed>";
	}

	/** {@inheritDoc} */
	@Override
	public T getObject() {
		return modelElement;
	}

	/** {@inheritDoc} */
	@Override
	public String toString() {
		return getName();
	}
}
