/*--------------------------------------------------------------------------+
$Id: ChannelAdapterWeightedEdge.java 2419 2016-03-04 18:05:52Z diewald $
|                                                                          |
| Copyright 2015 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.sysmodel.arch.taskgraph;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ISignalAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITransmissionUnitAdapter;
import org.jgrapht.graph.DefaultWeightedEdge;

/**
 * Edge specification class for underlying graph implementation.
 * 
 * @author diewald
 * @author $Author: diewald $
 * @version $Rev: 2419 $
 * @ConQAT.Rating RED Hash: 6EDF7F1A74B9CF6C7F01D64071FEC3DB
 */
public class ChannelAdapterWeightedEdge extends DefaultWeightedEdge {
	/** {@link ISignalAdapter} represented by this edge. */
	ISignalAdapter<?> channel;

	/** {@link ITransmissionUnitAdapter} to which this {@link ISignalAdapter} is mapped. */
	ITransmissionUnitAdapter<?> transmissionUnit;

	/** Constructs a new {@link ChannelAdapterWeightedEdge} for a given {@link ISignalAdapter}. */
	public ChannelAdapterWeightedEdge(ISignalAdapter<?> channel) {
		this.channel = channel;
	}

	// TODO: Remove the linking of channels with transmission units. This approach is no longer
	// valid for multi-hop communication.
	/**
	 * Sets the {@link ITransmissionUnitAdapter} to which the {@link ISignalAdapter} represented by
	 * this {@link ChannelAdapterWeightedEdge} is mapped.
	 */
	public void setTransmissionUnit(ITransmissionUnitAdapter<?> transmissionUnit) {
		this.transmissionUnit = transmissionUnit;
	}

	/** Returns the {@link ISignalAdapter} associated with this edge. */
	public ISignalAdapter<?> getChannelAdapter() {
		return channel;
	}

	/** {@inheritDoc} */
	@Override
	protected double getWeight() {
		if(transmissionUnit != null) {
			return channel.getTransmissionTime(transmissionUnit);
		}

		return 0;
	}
}
