/*--------------------------------------------------------------------------+
$Id: IMappingEncoding.java 3885 2016-11-11 09:41:01Z diewald $
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.sysmodel.mapping;

import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IDeployableComponentAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IRequestAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IResourceAdapter;
import org.fortiss.af3.exploration.alg.service.IExplorationEncoding;

/**
 * Generalized interface for accessing Request->Resource mappings that are the result of a DSE.
 * 
 * @author diewald
 * @author $Author: diewald $
 * @version $Rev: 3885 $
 * @ConQAT.Rating YELLOW Hash: 9B94F8118D5063A0E06F0A615BFD1199
 */
public interface IMappingEncoding extends IExplorationEncoding {

	/**
	 * Returns the allocation of a specific {@link IRequestAdapter} to its {@link IMappingEntry}s.
	 */
	public Collection<IResourceAdapter<?>> getAllocatedResourcesFor(IRequestAdapter<?> request);

	/**
	 * Returns the collection of {@link IMappingEntry}s which represent the (multi-)allocation of
	 * the given {@link IDeployableComponentAdapter}s to {@link IExecutionUnitAdapter}s.
	 */
	public Collection<IMappingEntry> getIMappingEntriesOf(IDeployableComponentAdapter<?> component);

	/**
	 * Returns whether {@code this} {@link IMappingEncoding} contains the given
	 * {@link IMappingEntry}.
	 */
	public boolean containsEntry(IMappingEntry entry);

	/**
	 * Returns the collection of {@link IMappingEntry}s contained in {@code this}
	 * {@link IMappingEncoding}.
	 */
	public Collection<IMappingEntry> getIMappingEntries();

	/** Returns the collection of deployed {@link IDeployableComponentAdapter}s. */
	public Collection<IDeployableComponentAdapter<?>> getDeployedComponents();
}
