/*--------------------------------------------------------------------------+
$Id: IMessageRouter.java 3032 2016-06-16 18:13:12Z diewald $
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.sysmodel.mapping.comm;

import java.util.Collection;
import java.util.Map;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ICommunicationResourceAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IResourceAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.comm.Message;
import org.jgrapht.DirectedGraph;
import org.jgrapht.graph.DefaultEdge;

/**
 * Interface for entities that route messages on a physical/virtual platform.
 * 
 * @author diewald
 * @author $Author: diewald $
 * @version $Rev: 3032 $
 * @ConQAT.Rating RED Hash: FA5520FE767C8CFCBB8DB765DDCEE7A3
 */
public interface IMessageRouter {

	/**
	 * Calculates the message routes for each {@link Message} given in the {@link Collection} {#link
	 * messagesToRoute}.
	 * 
	 * @return {@link Map} that relates the Message to its route.
	 */
	Map<Message, DirectedGraph<IResourceAdapter<?>, DefaultEdge>> calculateMessageRoutes(
			Collection<Message> messagesToRoute);

	/**
	 * Returns the remaining {@link Message}/communication graphs for the given {@link Message},
	 * where ... denotes the already processed {@link ICommunicationResourceAdapter}.
	 */
	// TODO: replace the IResourceAdapter with an ICommunicationResourceAdapter.
	Map<IResourceAdapter<?>, DirectedGraph<IResourceAdapter<?>, DefaultEdge>>
			getRemainingMessageGraphs(Message message, IResourceAdapter<?> processedResource,
					DirectedGraph<IResourceAdapter<?>, DefaultEdge> messageRoute);

	/**
	 * Calculates the route of a message from the source to given receivers. The route is given as a
	 * {@link DirectedGraph} whose leaves are the target {@link IResourceAdapter}s.
	 */
	DirectedGraph<IResourceAdapter<?>, DefaultEdge> calculateMessageRoute(
			IResourceAdapter<?> source, Collection<IResourceAdapter<?>> receiverResources);

	/** Returns the route which the given {@link Message} must traverse to deliver its content. */
	DirectedGraph<IResourceAdapter<?>, DefaultEdge> getRouteOf(Message msg);
}
