/*--------------------------------------------------------------------------+
$Id: ScheduleRuntimeException.java 2403 2016-03-03 17:10:02Z diewald $
|                                                                          |
| Copyright 2015 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.sysmodel.sched;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.solution.StrictTTSchedule;

/**
 * {@link RuntimeException} that is specific for errors in the scheduling. It allows to obtain the
 * faulty {@link StrictTTSchedule} via getter method such that it can be displayed in the schedule
 * plotter.
 * <p>
 * Since Opt4J does not allow to throw exceptions in the decoding phase, the only "clean" way to
 * hand faulty {@link StrictTTSchedule}s to the UI is using {@link RuntimeException}s.
 * 
 * @author diewald
 * @author $Author: diewald $
 * @version $Rev: 2403 $
 * @ConQAT.Rating YELLOW Hash: F59DECC44AC18F346664D4DF462D81AB
 */
public class ScheduleRuntimeException extends RuntimeException {

	/** The associated {@link StrictTTSchedule} of this exception. */
	private StrictTTSchedule<?, ?> schedule;

	/** Constructor. */
	public ScheduleRuntimeException(StrictTTSchedule<?, ?> schedule) {
		super();
		this.schedule = schedule;
	}

	/**
	 * Returns the (invalid) {@link StrictTTSchedule} that is which is associated with this
	 * exception and provides the given message.
	 */
	public ScheduleRuntimeException(StrictTTSchedule<?, ?> schedule, String message) {
		super(message);
		this.schedule = schedule;
	}

	/**
	 * Returns the (invalid) {@link StrictTTSchedule} that is which is associated with this
	 * exception.
	 */
	public StrictTTSchedule<?, ?> getSchedule() {
		return schedule;
	}

}
