/*--------------------------------------------------------------------------+
$Id: FailSilentEDFScheduler.java 3032 2016-06-16 18:13:12Z diewald $
|                                                                          |
| Copyright 2014 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.sysmodel.sched.proc;

import java.util.Map;

import org.apfloat.Apcomplex;
import org.apfloat.Apfloat;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskgraph.InstantiatedAcyclicTaskGraphEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.FailSilentTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.FailSilentTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.solution.StrictTTSchedule;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.solution.TimeSlot;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IDeployableComponentAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.fortiss.af3.exploration.model.ExplorationSpecification;

/**
 * Base class for EDF task schedulers
 * 
 * @author huang, barner
 * @author $Author: diewald $
 * @version $Rev: 3032 $
 * @ConQAT.Rating RED Hash: FA94B385E6898EA117C43DFF7A90982C
 */
public class FailSilentEDFScheduler extends
		EDFSchedulerBase<FailSilentTaskMappingEntry, FailSilentTaskMappingEncoding> {

	/** Constructs a simple EDF scheduler for a given exploration run */
	public FailSilentEDFScheduler(SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter,
			ExplorationSpecification expSpec) {
		super(systemModelAdapter, expSpec);
	}

	/**
	 * Returns the execution time for the current
	 * {@link org.fortiss.af3.exploration.alg.dse.sysmodel.sched.proc.EDFSchedulerBase.SchedulingObject}
	 * .
	 */
	@Override
	protected Apfloat getExecutionTime(TaskMappingEncoding<? extends TaskMappingEntry> encoding,
			InstantiatedAcyclicTaskGraphEncoding<FailSilentTaskMappingEntry, ?> acycItgEnc,
			IDeployableComponentAdapter<?> deployableComponent, FailSilentTaskMappingEntry entry) {
		Apfloat executiontime =
				new Apfloat(deployableComponent.getWcet(entry.getTarget()), DOUBLE_PRECISION);
		return executiontime;
	}

	/** {@inheritDoc} */
	@Override
	protected Apfloat registerIncomingMessagesWithSchedule(FailSilentTaskMappingEncoding encoding,
			InstantiatedAcyclicTaskGraphEncoding<FailSilentTaskMappingEntry, ?> acycItgEnc,
			StrictTTSchedule<FailSilentTaskMappingEntry, FailSilentTaskMappingEncoding> schedule,
			Map<FailSilentTaskMappingEntry, TimeSlot> mappingEntryToTimeSlot,
			IDeployableComponentAdapter<?> deployableComponent, Apfloat starttime,
			IExecutionUnitAdapter<?> deploymentTarget, long iteration, TimeSlot slot) {
		// TODO Auto-generated method stub
		return Apcomplex.ZERO;
	}
}
