/*--------------------------------------------------------------------------+
$Id: InvalidPlatformModelException.java 3425 2016-08-31 07:44:08Z barner $
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.exception;

import org.fortiss.af3.platform.model.IPlatformArchitectureElement;
import org.fortiss.af3.platform.model.PlatformArchitecture;
import org.fortiss.af3.platform.model.PlatformConnectorUnit;

/**
 * Exception to be thrown whenever an input {@link PlatformArchitecture} model to the DSE has been
 * detected.
 * 
 * @author diewald
 * @author $Author: barner $
 * @version $Rev: 3425 $
 * @ConQAT.Rating YELLOW Hash: CE0822F07AC2CF9DB2CE941650C88FD3
 */
public class InvalidPlatformModelException extends Exception {

	/** Constructor. */
	public InvalidPlatformModelException(IPlatformArchitectureElement element, String msg) {
		super("A PlatformArchitecture contains the invalid element " + element.getName() + "(id: " +
				element.getId() + "). Details:\n" + msg);
	}

	/** Constructor that stacks an additional {@link Exception}. */
	public InvalidPlatformModelException(IPlatformArchitectureElement element, String msg,
			Exception e) {
		super("A PlatformArchitecture contains the invalid element " + element.getName() + "(id: " +
				element.getId() + "). Details:\n" + msg, e);
	}

	/** Constructor. */
	public InvalidPlatformModelException(PlatformConnectorUnit element, String msg) {
		super("A PlatformArchitecture contains the invalid element " + element.getName() + "(id: " +
				element.getId() + "). Details:\n" + msg);
	}

	/** Constructor that stacks an additional {@link Exception}. */
	public InvalidPlatformModelException(PlatformConnectorUnit element, String msg, Exception e) {
		super("A PlatformArchitecture contains the invalid element " + element.getName() + "(id: " +
				element.getId() + "). Details:\n" + msg, e);
	}
}
