/*--------------------------------------------------------------------------+
$Id: codetemplates.xml 1 2011-01-01 00:00:01Z hoelzl $
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.guava;

import java.util.Collection;

/**
 * {@link IAssociativeElement} define corresponding {@link IAssociativeElement}s of another DSE
 * process stage.
 * 
 * @author diewald
 * @author $Author: hoelzl $
 * @version $Rev: 18709 $
 * @ConQAT.Rating RED Hash:
 */
public interface IAssociativeElement<T extends IAssociativeElement<?>> {

	/**
	 * Adds the given {@code entry} to the set of associated {@link IAssociativeElement}s of
	 * {@code this} {@link IAssociativeElement}.
	 * 
	 * @param entry
	 *            The {@link IAssociativeElement} to associate with.
	 */
	void addAssociatedElement(T entry);

	/**
	 * Removes the given {@code entry} to the set of associated {@link IAssociativeElement}s of
	 * {@code this} {@link IAssociativeElement}.
	 * 
	 * @param entry
	 *            The {@link IAssociativeElement} to remove if any is present.
	 */
	void rmAssociatedElement(T entry);

	/**
	 * Returns the set of associated {@link IAssociativeElement}s of {@code this}
	 * {@link IAssociativeElement}.
	 */
	Collection<T> getAssociatedElement();

	/**
	 * Returns the associated {@link IAssociativeElement} of {@code this}
	 * {@link IAssociativeElement} which has the given type.
	 */
	<S extends T> S getAssociatedElement(Class<S> type);
}
