/*--------------------------------------------------------------------------+
$Id: codetemplates.xml 1 2011-01-01 00:00:01Z hoelzl $
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.guava;

import org.conqat.lib.commons.collections.ImmutablePair;
import org.conqat.lib.commons.collections.Pair;

/**
 * {@link Pair} whose first entry is an actual object whereas the second entry defines its type. Its
 * hash value is calculated from the hash codes of its entries.
 * The main purpose of the {@link TypedPair} is to support {@link TypedBiMap}s.
 * 
 * @author diewald
 * @author $Author: hoelzl $
 * @version $Rev: 18709 $
 * @ConQAT.Rating RED Hash: FF13396A37CA9800CBE10A9E71DCB868
 */
public class TypedPair<S, T> extends Pair<S, Class<T>> {

	/** Constructor. */
	public TypedPair(ImmutablePair<S, Class<T>> p) {
		super(p);
	}

	/** Constructor. */
	public TypedPair(S first, Class<T> second) {
		super(first, second);
	}

	/** {@inheritDoc} */
	@Override
	public int hashCode() {
		return first.hashCode() + second.hashCode();
	}
}
