/*--------------------------------------------------------------------------+
$Id: PrototypeProvider.java 3223 2016-07-06 14:11:30Z diewald $
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.prototypes;

import static org.fortiss.af3.exploration.alg.util.DesignSpaceExplorationModelElementFactory.createDseSpecification;
import static org.fortiss.af3.exploration.alg.util.DesignSpaceExplorationModelElementFactory.createExplorationManager;

import org.fortiss.af3.exploration.moea.model.DseSpecification;
import org.fortiss.af3.exploration.moea.model.ExplorationManager;
import org.fortiss.tooling.kernel.extension.base.PrototypeProviderBase;

/**
 * Prototype provider for design space exploration object.
 * 
 * @author diewald
 * @author $Author: diewald $
 * @version $Rev: 3223 $
 * @ConQAT.Rating GREEN Hash: CB0DF6FCCFF7C9D25474224C1364C074
 */
public class PrototypeProvider extends PrototypeProviderBase {

	/** Stores the category to display for these prototypes. */
	protected final static String CATEGORY_NAME = "Design Space Exploration";

	/** {@inheritDoc} */
	@Override
	protected void registerPrototypes() {
		setupExplorationManagerPrototypes();
		setupDseSpecificationPrototypes();
	}

	/**
	 * Defines prototypes for {@link ExplorationManager}s which are used to manage DSE
	 * models.
	 */
	private void setupExplorationManagerPrototypes() {
		registerPrimaryPrototype("Design Space Exploration",
				createExplorationManager("Design Space Exploration"), CATEGORY_NAME);
	}

	/**
	 * Defines prototypes for {@link DseSpecification}s which are used to create and configure
	 * DSE models.
	 */
	private void setupDseSpecificationPrototypes() {
		registerPrimaryPrototype("DSE Specification", createDseSpecification("DSE Specification"),
				CATEGORY_NAME);
	}
}
