/*--------------------------------------------------------------------------+
$Id: codetemplates.xml 1 2011-01-01 00:00:01Z hoelzl $
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.service;

import java.util.Collection;
import java.util.Set;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.evaluate.ComposableEvaluator;

/**
 * {@link IExplorationService} interface to register evaluators for the DSE. See
 * {@link IExplorationTargetEvaluator} for the interface that needs to be implemented by concrete
 * evaluator modules.
 * 
 * @author diewald
 * @author $Author: hoelzl $
 * @version $Rev: 18709 $
 * @ConQAT.Rating YELLOW Hash: 051B57E0A2EA559A454139D78407FF8C
 */
public interface IExplorationEvaluatationService<T extends IExplorationModule> extends
		IExplorationService<T> {

	/**
	 * Returns the collection of input model types that are required to execute the given set of
	 * {@link ComposableEvaluator}s. The models corresponding to these types can be retrieved via
	 * the {@link IExplorationTransformationService}.
	 * 
	 * @param evaluators
	 *            Set of evaluators that shall be executed.
	 */
	Collection<Class<?>> getRequiredTransformedModels(
			Set<IExplorationTargetEvaluator<?, ?, ?>> evaluators);
}
