/*--------------------------------------------------------------------------+
$Id: codetemplates.xml 1 2011-01-01 00:00:01Z hoelzl $
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.service;

import java.util.Collection;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.evaluate.ComposableEvaluator;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.FlatPhenotypeMap;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.fortiss.af3.exploration.model.ExplorationTarget;
import org.fortiss.tooling.kernel.service.base.IEObjectAware;
import org.opt4j.core.Value;

import com.google.common.collect.ClassToInstanceMap;

/**
 * Interface to be implemented by {@link IExplorationModule}s that perform evaluations of
 * individuals (= candidate solutions). These are generated during the execution of the DSE.
 * 
 * @author diewald
 * @author $Author: hoelzl $
 * @version $Rev: 18709 $
 * @ConQAT.Rating YELLOW Hash: 37C9FF948C80C9486868B9149D2A1D6D
 */
public interface IExplorationTargetEvaluator<P, RT, S extends ExplorationTarget<RT>> extends
		IEObjectAware<S>, IExplorationModule {

	/**
	 * Evaluates a phenotype w.r.t. a given objective dimension
	 * 
	 * @param transformedModels
	 *            TODO
	 */
	public Value<?> evaluateGoal(P phenotype, FlatPhenotypeMap<Phenotype> phenotypeTypeMap,
			ClassToInstanceMap<EObject> transformedModels);

	/** Validate if objective has been correctly specified. */
	public boolean validateGoalSpecification();

	/**
	 * Returns the set of input types (classes) which are required for the enclosed
	 * {@link ComposableEvaluator}.
	 */
	Collection<Class<?>> getRequiredInputTypes();

	/** Returns the phenotype type for which this evaluator is registered. */
	// Only needed until the automatic model transformation is finished
	Class<?> getPrimaryInputType();

	/**
	 * Creates an instance of {@code this} {@link IExplorationTargetEvaluator}. The
	 * {@code systemModelAdapter} and the {@code explorationTarget} are passed from the DSE
	 * framework.
	 * 
	 * @param systemModelAdapter
	 *            Internal representation of the base models.
	 * @param explorationTarget
	 *            Specifies the concrete exploration target including the parameters.
	 * @return A new instance of {@code this} {@link IExplorationTargetEvaluator}.
	 * @throws InstantiationException
	 *             If the instantiation of a new {@link IExplorationTargetEvaluator} fails.
	 */
	IExplorationTargetEvaluator<P, RT, S> createEvaluator(
			SystemModelAdapter<?, ?, ?, ?, ?, ?> systemModelAdapter, S explorationTarget)
			throws InstantiationException;
}
