/*--------------------------------------------------------------------------+
$Id: ExplorationEcoreUtils.java 3490 2016-09-13 11:39:58Z diewald $
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.util;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil.Copier;
import org.fortiss.tooling.kernel.utils.EcoreUtils;

/**
 * Utility methods for the interaction with EMF models (candidates to be moved into the
 * {@link EcoreUtils} class).
 * 
 * @author diewald
 * @author $Author: diewald $
 * @version $Rev: 3490 $
 * @ConQAT.Rating RED Hash: DA40D522ACC49188F475BC7B38CEE7E8
 */
public class ExplorationEcoreUtils {

	/** Prevent instantiation. */
	private ExplorationEcoreUtils() {
	}

	/**
	 * Returns a self-contained copy of the given <code>eObject</code>.
	 * Here, the copier that performs the copy operation must be given. The copier (which is a
	 * LinkedHashMap) can be further utilized to obtain information about the copied elements.
	 * 
	 * @param eObject
	 *            the object to copy.
	 * @param copier
	 *            This copier will be used to perform the actual copy operation.
	 * @return the copy.
	 * @see Copier
	 */
	public static <T extends EObject> T copy(T eObject, Copier copier) {
		EObject result = copier.copy(eObject);
		copier.copyReferences();

		@SuppressWarnings("unchecked") T t = (T)result;
		return t;
	}
}
