/*--------------------------------------------------------------------------+
$Id: LocationConstraintStaticImpl.java 3175 2016-07-04 15:25:48Z diewald $
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.model.location.impl;

import org.fortiss.af3.exploration.dsl.model.expressions.ElementExpression;
import org.fortiss.af3.exploration.dsl.model.expressions.Expression;
import org.fortiss.af3.exploration.dsl.model.operators.LocationOp;
import org.fortiss.af3.exploration.dsl.model.patterns.LocationPattern;
import org.fortiss.af3.exploration.model.location.LocationConstraint;
import org.fortiss.tooling.base.model.element.IModelElement;

/**
 * Static method implementations for {@link LocationConstraint}s.
 * 
 * @author diewald
 * @author $Author: diewald $
 * @version $Rev: 3175 $
 * @ConQAT.Rating YELLOW Hash: D766186F5D9A8C9C7C06EFD2F8DCA83F
 */
public class LocationConstraintStaticImpl {

	/** Static implementation of the {@link LocationConstraint#getLeftModelElement()} method. */
	public static <LHT extends IModelElement, RHT extends IModelElement> LHT getLeftModelElement(
			LocationConstraint<LHT, RHT> constr) {
		if(constr == null) {
			return null;
		}
		LocationOp<LHT, RHT> locOp = getLocationOperator(constr);
		if(locOp != null) {
			// TODO: Write helper method for ElementExpressions.
			ElementExpression<LHT> leftExpr = locOp.getLeft();
			if(leftExpr != null) {
				return leftExpr.getModelElement();
			}
		}
		return null;
	}

	/** Static implementation of the {@link LocationConstraint#getRightModelElement()} method. */
	public static <LHT extends IModelElement, RHT extends IModelElement> RHT getRightModelElement(
			LocationConstraint<LHT, RHT> constr) {
		if(constr == null) {
			return null;
		}
		LocationOp<LHT, RHT> locOp = getLocationOperator(constr);
		if(locOp != null) {
			// TODO: Write helper method for ElementExpressions.
			ElementExpression<RHT> rightExpr = locOp.getRight();
			if(rightExpr != null) {
				return rightExpr.getModelElement();
			}
		}
		return null;
	}

	/**
	 * Returns the {@link LocationOp} of the given {@link LocationConstraint}, or{@code null} if noe
	 * is found.
	 */
	@SuppressWarnings("unchecked")
	public static
			<LHT extends IModelElement, RHT extends IModelElement, LO extends LocationOp<LHT, RHT>>
			LO getLocationOperator(LocationConstraint<LHT, RHT> locConstraint) {
		if(locConstraint == null) {
			return null;
		}
		Expression expr = locConstraint.getExpression();
		if(expr instanceof LocationPattern) {
			Expression lpExpr = ((LocationPattern<?, ?>)expr).getExpression();
			if(lpExpr instanceof LocationOp) {
				return (LO)lpExpr;
			}
		}
		return null;
	}
}
