/*--------------------------------------------------------------------------+
$Id: PeriodConstraintStaticImpl.java 3175 2016-07-04 15:25:48Z diewald $
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.model.time.impl;

import org.fortiss.af3.component.model.Component;
import org.fortiss.af3.exploration.dsl.model.expressions.ArithmeticExpression;
import org.fortiss.af3.exploration.dsl.model.expressions.ConstraintExpression;
import org.fortiss.af3.exploration.dsl.model.patterns.ElementProperty;
import org.fortiss.af3.exploration.dsl.model.patterns.SinglePropValueComparison;
import org.fortiss.af3.exploration.dsl.model.types.NumberLiteral;
import org.fortiss.af3.exploration.model.time.PeriodConstraint;
import org.fortiss.tooling.base.model.element.IModelElement;

/**
 * Static method implementations for {@link PeriodConstraint}s.
 * 
 * @author diewald
 * @author $Author: diewald $
 * @version $Rev: 3175 $
 * @ConQAT.Rating YELLOW Hash: 3E1ECF0408C90342BDC4A73B8E80845F
 */
public class PeriodConstraintStaticImpl {

	/** Static implementation of the {@link PeriodConstraint#getComponent()} method. */
	public static Component getComponent(PeriodConstraint constr) {
		if(constr == null) {
			return null;
		}
		ElementProperty<Component, Double> elemProp = getElementProperty(constr);
		if(elemProp != null) {
			return elemProp.getModelElement();
		}
		return null;
	}

	/** Static implementation of the {@link PeriodConstraint#setComponent(Component)} method. */
	public static void setComponent(PeriodConstraint constr, Component component) {
		if(constr == null) {
			return;
		}
		ElementProperty<Component, Double> elemProp = getElementProperty(constr);
		if(elemProp != null) {
			elemProp.setModelElement(component);
		}
	}

	/** Static implementation of the {@link PeriodConstraint#getPeriod()} method. */
	public static Double getPeriod(PeriodConstraint constr) {
		if(constr == null) {
			return null;
		}
		NumberLiteral<Double> periodLiteral = getNumberLiteral(constr);
		if(periodLiteral != null) {
			return periodLiteral.getValue();
		}
		return null;
	}

	/** Static implementation of the {@link PeriodConstraint#setPeriod(Double)} method. */
	public static void setPeriod(PeriodConstraint constr, Double period) {
		if(constr == null) {
			return;
		}
		NumberLiteral<Double> periodLiteral = getNumberLiteral(constr);
		if(periodLiteral != null) {
			periodLiteral.setValue(period);
		}
	}

	/**
	 * Returns the {@link ElementProperty} which is contained in {@code this}
	 * {@link PeriodConstraint}.
	 */
	// Type parameters are enforced by construction.
	@SuppressWarnings("unchecked")
	public static <ME extends IModelElement, T> ElementProperty<ME, T> getElementProperty(
			PeriodConstraint constr) {
		if(constr == null) {
			return null;
		}
		if(constr.getExpression() instanceof ConstraintExpression) {
			ConstraintExpression<?> expr = (ConstraintExpression<?>)constr.getExpression();
			if(expr instanceof SinglePropValueComparison) {
				SinglePropValueComparison<Double, Component> svCompExpr =
						(SinglePropValueComparison<Double, Component>)expr.getExpression();
				if(svCompExpr != null) {
					ArithmeticExpression<?> leftExpr = svCompExpr.getLeft();
					if(leftExpr instanceof ElementProperty) {
						return((ElementProperty<ME, T>)leftExpr);
					}
				}
			}
		}

		return null;
	}

	/**
	 * Returns the {@link NumberLiteral} which is contained in {@code this} {@link PeriodConstraint}
	 * .
	 */
	// Type parameters are enforced by construction.
	@SuppressWarnings("unchecked")
	public static <T> NumberLiteral<T> getNumberLiteral(PeriodConstraint constr) {
		if(constr == null) {
			return null;
		}
		if(constr.getExpression() instanceof ConstraintExpression<?>) {
			ConstraintExpression<?> expr = (ConstraintExpression<?>)constr.getExpression();
			if(expr instanceof SinglePropValueComparison) {
				SinglePropValueComparison<Double, Component> svCompExpr =
						(SinglePropValueComparison<Double, Component>)expr.getExpression();
				if(svCompExpr != null) {
					ArithmeticExpression<?> rightExpr = svCompExpr.getRight();
					if(rightExpr instanceof NumberLiteral) {
						return (NumberLiteral<T>)rightExpr;
					}
				}
			}
		}

		return null;
	}
}
